@@ -13,9 +13,11 @@ from .isa import thumb_isa, ThumbToken
 
 # Instructions:
 
+
 class ThumbInstruction(Instruction):
     """ Base of all thumb instructions.
     """
+
     tokens = [ThumbToken]
     isa = thumb_isa
 
@@ -25,7 +27,7 @@ class LongThumbInstruction(ThumbInstruction):
 
 
 class nop_ins(ThumbInstruction):
-    syntax = Syntax(['nop'])
+    syntax = Syntax(["nop"])
 
     def encode(self):
         return bytes()
@@ -33,10 +35,12 @@ class nop_ins(ThumbInstruction):
 
 # Memory related
 
+
 class LS_imm5_base(ThumbInstruction):
     """ ??? Rt, [Rn, imm5] """
-    rn = Operand('rn', LowArmRegister, read=True)
-    imm5 = Operand('imm5', int)
+
+    rn = Operand("rn", LowArmRegister, read=True)
+    imm5 = Operand("imm5", int)
 
     def encode(self):
         assert self.imm5 % 4 == 0
@@ -54,25 +58,49 @@ class LS_imm5_base(ThumbInstruction):
 
 
 class Str2(LS_imm5_base):
-    rt = Operand('rt', LowArmRegister, read=True)
-    syntax = Syntax([
-        'str', ' ', rt, ',', '[', LS_imm5_base.rn,
-        ',', ' ', LS_imm5_base.imm5, ']'])
+    rt = Operand("rt", LowArmRegister, read=True)
+    syntax = Syntax(
+        [
+            "str",
+            " ",
+            rt,
+            ",",
+            "[",
+            LS_imm5_base.rn,
+            ",",
+            " ",
+            LS_imm5_base.imm5,
+            "]",
+        ]
+    )
     opcode = 0xC
 
 
 class Ldr2(LS_imm5_base):
-    rt = Operand('rt', LowArmRegister, write=True)
-    syntax = Syntax([
-        'ldr', ' ', rt, ',', ' ', '[', LS_imm5_base.rn,
-        ',', ' ', LS_imm5_base.imm5, ']'])
+    rt = Operand("rt", LowArmRegister, write=True)
+    syntax = Syntax(
+        [
+            "ldr",
+            " ",
+            rt,
+            ",",
+            " ",
+            "[",
+            LS_imm5_base.rn,
+            ",",
+            " ",
+            LS_imm5_base.imm5,
+            "]",
+        ]
+    )
     opcode = 0xD
 
 
 class LS_byte_imm5_base(ThumbInstruction):
     """ ??? Rt, [Rn, imm5] """
-    rn = Operand('rn', LowArmRegister, read=True)
-    imm5 = Operand('imm5', int)
+
+    rn = Operand("rn", LowArmRegister, read=True)
+    imm5 = Operand("imm5", int)
 
     def encode(self):
         assert self.rn.num < 8
@@ -89,28 +117,52 @@ class LS_byte_imm5_base(ThumbInstruction):
 
 
 class Strb(LS_byte_imm5_base):
-    rt = Operand('rt', LowArmRegister, read=True)
-    syntax = Syntax([
-        'strb', ' ', rt, ',', ' ', '[', LS_byte_imm5_base.rn, ',', ' ',
-        LS_byte_imm5_base.imm5, ']'])
+    rt = Operand("rt", LowArmRegister, read=True)
+    syntax = Syntax(
+        [
+            "strb",
+            " ",
+            rt,
+            ",",
+            " ",
+            "[",
+            LS_byte_imm5_base.rn,
+            ",",
+            " ",
+            LS_byte_imm5_base.imm5,
+            "]",
+        ]
+    )
     opcode = 0xE
 
 
 class Ldrb(LS_byte_imm5_base):
-    rt = Operand('rt', LowArmRegister, write=True)
-    syntax = Syntax([
-        'ldrb', ' ', rt, ',', ' ', '[', LS_byte_imm5_base.rn, ',', ' ',
-        LS_byte_imm5_base.imm5, ']'])
+    rt = Operand("rt", LowArmRegister, write=True)
+    syntax = Syntax(
+        [
+            "ldrb",
+            " ",
+            rt,
+            ",",
+            " ",
+            "[",
+            LS_byte_imm5_base.rn,
+            ",",
+            " ",
+            LS_byte_imm5_base.imm5,
+            "]",
+        ]
+    )
     opcode = 0b01111
 
 
 class Strh(ThumbInstruction):
     """ Strh Rt, [Rn, imm5] """
-    rt = Operand('rt', LowArmRegister, read=True)
-    rn = Operand('rn', LowArmRegister, read=True)
-    imm5 = Operand('imm5', int)
-    syntax = Syntax([
-        'strh', ' ', rt, ',', ' ', '[', rn, ',', ' ', imm5, ']'])
+
+    rt = Operand("rt", LowArmRegister, read=True)
+    rn = Operand("rn", LowArmRegister, read=True)
+    imm5 = Operand("imm5", int)
+    syntax = Syntax(["strh", " ", rt, ",", " ", "[", rn, ",", " ", imm5, "]"])
 
     def encode(self):
         opcode = 0x10
@@ -129,11 +181,11 @@ class Strh(ThumbInstruction):
 
 class Ldrh(ThumbInstruction):
     """ Ldrh Rt, [Rn, imm5] """
-    rt = Operand('rt', LowArmRegister, write=True)
-    rn = Operand('rn', LowArmRegister, read=True)
-    imm5 = Operand('imm5', int)
-    syntax = Syntax([
-        'ldrh', ' ', rt, ',', ' ', '[', rn, ',', ' ', imm5, ']'])
+
+    rt = Operand("rt", LowArmRegister, write=True)
+    rn = Operand("rn", LowArmRegister, read=True)
+    imm5 = Operand("imm5", int)
+    syntax = Syntax(["ldrh", " ", rt, ",", " ", "[", rn, ",", " ", imm5, "]"])
 
     def encode(self):
         opcode = 0x11
@@ -151,7 +203,7 @@ class Ldrh(ThumbInstruction):
 
 
 class ls_sp_base_imm8(ThumbInstruction):
-    offset = Operand('offset', int)
+    offset = Operand("offset", int)
 
     def encode(self):
         rt = self.rt.num
@@ -164,9 +216,10 @@ class ls_sp_base_imm8(ThumbInstruction):
 
 class Ldr3(ThumbInstruction):
     """ ldr Rt, LABEL, load value from pc relative position """
-    rt = Operand('rt', LowArmRegister, write=True)
-    label = Operand('label', str)
-    syntax = Syntax(['ldr', ' ', rt, ',', ' ', label])
+
+    rt = Operand("rt", LowArmRegister, write=True)
+    label = Operand("label", str)
+    syntax = Syntax(["ldr", " ", rt, ",", " ", label])
 
     def relocations(self):
         return [Lit8Relocation(self.label)]
@@ -181,26 +234,52 @@ class Ldr3(ThumbInstruction):
 
 class Ldr1(ls_sp_base_imm8):
     """ ldr Rt, [SP, imm8] """
-    rt = Operand('rt', LowArmRegister, write=True)
+
+    rt = Operand("rt", LowArmRegister, write=True)
     opcode = 0x98
     syntax = Syntax(
-        ['ldr', ' ', rt, ',', ' ', '[', 'sp', ',',
-         ' ', ls_sp_base_imm8.offset, ']'])
+        [
+            "ldr",
+            " ",
+            rt,
+            ",",
+            " ",
+            "[",
+            "sp",
+            ",",
+            " ",
+            ls_sp_base_imm8.offset,
+            "]",
+        ]
+    )
 
 
 class Str1(ls_sp_base_imm8):
     """ str Rt, [SP, imm8] """
-    rt = Operand('rt', LowArmRegister, read=True)
+
+    rt = Operand("rt", LowArmRegister, read=True)
     opcode = 0x90
     syntax = Syntax(
-        ['str', ' ', rt, ',', ' ', '[', 'sp', ',', ' ',
-         ls_sp_base_imm8.offset, ']'])
+        [
+            "str",
+            " ",
+            rt,
+            ",",
+            " ",
+            "[",
+            "sp",
+            ",",
+            " ",
+            ls_sp_base_imm8.offset,
+            "]",
+        ]
+    )
 
 
 class Adr(ThumbInstruction):
-    rd = Operand('rd', LowArmRegister, write=True)
-    label = Operand('label', str)
-    syntax = Syntax(['adr', rd, ',', label])
+    rd = Operand("rd", LowArmRegister, write=True)
+    label = Operand("label", str)
+    syntax = Syntax(["adr", rd, ",", label])
 
     def relocations(self):
         return [Lit8Relocation(self.label)]
@@ -215,10 +294,11 @@ class Adr(ThumbInstruction):
 
 class Mov3(ThumbInstruction):
     """ mov Rd, imm8, move immediate value into register """
-    opcode = 4   # 00100 Rd(3) imm8
-    rd = Operand('rd', LowArmRegister, write=True)
-    imm = Operand('imm', int)
-    syntax = Syntax(['mov', ' ', rd, ',', ' ', imm])
+
+    opcode = 4  # 00100 Rd(3) imm8
+    rd = Operand("rd", LowArmRegister, write=True)
+    imm = Operand("imm", int)
+    syntax = Syntax(["mov", " ", rd, ",", " ", imm])
 
     def encode(self):
         tokens = self.get_tokens()
@@ -228,13 +308,14 @@ class Mov3(ThumbInstruction):
         tokens[0][11:16] = self.opcode
         return tokens[0].encode()
 
+
 # Arithmatics:
 
 
 class regregimm3_base(ThumbInstruction):
-    rd = Operand('rd', LowArmRegister, write=True)
-    rn = Operand('rn', LowArmRegister, read=True)
-    imm3 = Operand('imm3', int)
+    rd = Operand("rd", LowArmRegister, write=True)
+    rn = Operand("rn", LowArmRegister, read=True)
+    imm3 = Operand("imm3", int)
 
     def encode(self):
         assert self.imm3 < 8
@@ -249,25 +330,46 @@ class regregimm3_base(ThumbInstruction):
 
 class AddImm(regregimm3_base):
     """ add Rd, Rn, imm3 """
-    syntax = Syntax([
-        'add', ' ', regregimm3_base.rd, ',', ' ', regregimm3_base.rn, ',',
-        regregimm3_base.imm3])
+
+    syntax = Syntax(
+        [
+            "add",
+            " ",
+            regregimm3_base.rd,
+            ",",
+            " ",
+            regregimm3_base.rn,
+            ",",
+            regregimm3_base.imm3,
+        ]
+    )
     opcode = 0b0001110
 
 
 class SubImm(regregimm3_base):
     """ sub Rd, Rn, imm3 """
-    syntax = Syntax([
-        'sub', ' ', regregimm3_base.rd, ',', ' ', regregimm3_base.rn, ',',
-        regregimm3_base.imm3])
+
+    syntax = Syntax(
+        [
+            "sub",
+            " ",
+            regregimm3_base.rd,
+            ",",
+            " ",
+            regregimm3_base.rn,
+            ",",
+            regregimm3_base.imm3,
+        ]
+    )
     opcode = 0b0001111
 
 
 class regregreg_base(ThumbInstruction):
     """ ??? Rd, Rn, Rm """
-    rd = Operand('rd', LowArmRegister, write=True)
-    rn = Operand('rn', LowArmRegister, read=True)
-    rm = Operand('rm', LowArmRegister, read=True)
+
+    rd = Operand("rd", LowArmRegister, write=True)
+    rn = Operand("rn", LowArmRegister, read=True)
+    rm = Operand("rm", LowArmRegister, read=True)
 
     def encode(self):
         at = ThumbToken()
@@ -281,24 +383,45 @@ class regregreg_base(ThumbInstruction):
 
 
 class Add3(regregreg_base):
-    syntax = Syntax([
-        'add', ' ', regregreg_base.rd, ',', ' ', regregreg_base.rn, ',', ' ',
-        regregreg_base.rm])
+    syntax = Syntax(
+        [
+            "add",
+            " ",
+            regregreg_base.rd,
+            ",",
+            " ",
+            regregreg_base.rn,
+            ",",
+            " ",
+            regregreg_base.rm,
+        ]
+    )
     opcode = 0b0001100
 
 
 class Sub3(regregreg_base):
-    syntax = Syntax([
-        'sub', ' ', regregreg_base.rd, ',', ' ', regregreg_base.rn, ',', ' ',
-        regregreg_base.rm])
+    syntax = Syntax(
+        [
+            "sub",
+            " ",
+            regregreg_base.rd,
+            ",",
+            " ",
+            regregreg_base.rn,
+            ",",
+            " ",
+            regregreg_base.rm,
+        ]
+    )
     opcode = 0b0001101
 
 
 class Mov2(ThumbInstruction):
     """ mov rd, rm (all registers, also > r7 """
-    rd = Operand('rd', ArmRegister, write=True)
-    rm = Operand('rm', ArmRegister, read=True)
-    syntax = Syntax(['mov', ' ', rd, ',', ' ', rm])
+
+    rd = Operand("rd", ArmRegister, write=True)
+    rm = Operand("rm", ArmRegister, read=True)
+    syntax = Syntax(["mov", " ", rd, ",", " ", rm])
 
     def encode(self):
         tokens = self.get_tokens()
@@ -319,9 +442,10 @@ class Mul(ThumbInstruction):
         multiply Rn and Rm and store the result in Rd
         Rd and Rm are the same register.
     """
-    rn = Operand('rn', LowArmRegister, read=True)
-    rdm = Operand('rdm', LowArmRegister, read=True, write=True)
-    syntax = Syntax(['mul', ' ', rn, ',', ' ', rdm])
+
+    rn = Operand("rn", LowArmRegister, read=True)
+    rdm = Operand("rdm", LowArmRegister, read=True, write=True)
+    syntax = Syntax(["mul", " ", rn, ",", " ", rdm])
 
     def encode(self):
         tokens = self.get_tokens()
@@ -337,10 +461,11 @@ class Sdiv(LongThumbInstruction):
     """ Signed division.
         Encoding T1
     """
-    rd = Operand('rd', ArmRegister, write=True)
-    rn = Operand('rn', ArmRegister, read=True)
-    rm = Operand('rm', ArmRegister, read=True)
-    syntax = Syntax(['sdiv', ' ', rd, ',', ' ', rn, ',', ' ', rm])
+
+    rd = Operand("rd", ArmRegister, write=True)
+    rn = Operand("rn", ArmRegister, read=True)
+    rm = Operand("rm", ArmRegister, read=True)
+    syntax = Syntax(["sdiv", " ", rd, ",", " ", rn, ",", " ", rm])
 
     def encode(self):
         tokens = self.get_tokens()
@@ -356,6 +481,7 @@ class Sdiv(LongThumbInstruction):
 
 class regreg_base(ThumbInstruction):
     """ ??? Rdn, Rm """
+
     def encode(self):
         tokens = self.get_tokens()
         tokens[0].rd = self.rdn.num
@@ -366,29 +492,30 @@ class regreg_base(ThumbInstruction):
 
 
 def make_regreg(mnemonic, opcode):
-    rdn = Operand('rdn', LowArmRegister, write=True, read=True)
-    rm = Operand('rm', LowArmRegister, read=True)
-    syntax = Syntax([mnemonic, rdn, ',', rm])
-    members = {'syntax': syntax, 'rdn': rdn, 'rm': rm, 'opcode': opcode}
-    return type(mnemonic + '_ins', (regreg_base,), members)
+    rdn = Operand("rdn", LowArmRegister, write=True, read=True)
+    rm = Operand("rm", LowArmRegister, read=True)
+    syntax = Syntax([mnemonic, rdn, ",", rm])
+    members = {"syntax": syntax, "rdn": rdn, "rm": rm, "opcode": opcode}
+    return type(mnemonic + "_ins", (regreg_base,), members)
 
 
-And = make_regreg('and', 0b0100000000)
-Orr = make_regreg('orr', 0b0100001100)
-Eor = make_regreg('eor', 0b0100000001)
-Cmp = make_regreg('cmp', 0b0100001010)
-Lsl = make_regreg('lsl', 0b0100000010)
-Lsr = make_regreg('lsr', 0b0100000011)
-Asr = make_regreg('lsr', 0b0100000100)
-Rsb = make_regreg('rsb', 0b0100001001)
+And = make_regreg("and", 0b0100000000)
+Orr = make_regreg("orr", 0b0100001100)
+Eor = make_regreg("eor", 0b0100000001)
+Cmp = make_regreg("cmp", 0b0100001010)
+Lsl = make_regreg("lsl", 0b0100000010)
+Lsr = make_regreg("lsr", 0b0100000011)
+Asr = make_regreg("lsr", 0b0100000100)
+Rsb = make_regreg("rsb", 0b0100001001)
 
 
 class Cmp2(ThumbInstruction):
     """ cmp Rn, imm8 """
-    opcode = 5   # 00101
-    rn = Operand('rn', LowArmRegister, read=True)
-    imm = Operand('imm', int)
-    syntax = Syntax(['cmp', rn, ',', imm])
+
+    opcode = 5  # 00101
+    rn = Operand("rn", LowArmRegister, read=True)
+    imm = Operand("imm", int)
+    syntax = Syntax(["cmp", rn, ",", imm])
 
     def encode(self):
         tokens = self.get_tokens()
@@ -402,8 +529,8 @@ class Cmp2(ThumbInstruction):
 
 
 class B(ThumbInstruction):
-    target = Operand('target', str)
-    syntax = Syntax(['b', ' ', target])
+    target = Operand("target", str)
+    syntax = Syntax(["b", " ", target])
 
     def encode(self):
         h = (0b11100 << 11) | 0
@@ -418,8 +545,9 @@ class Bw(LongThumbInstruction):
     """ Encoding T4
         Same encoding as Bl, longer jumps are possible with this function!
     """
-    target = Operand('target', str)
-    syntax = Syntax(['bw', ' ', target])
+
+    target = Operand("target", str)
+    syntax = Syntax(["bw", " ", target])
 
     def encode(self):
         j1 = 1
@@ -438,8 +566,9 @@ class Bw(LongThumbInstruction):
 
 class Bl(LongThumbInstruction):
     """ Branch with link """
-    target = Operand('target', str)
-    syntax = Syntax(['bl', ' ', target])
+
+    target = Operand("target", str)
+    syntax = Syntax(["bl", " ", target])
 
     def encode(self):
         j1 = 1  # TODO: what do these mean?
@@ -459,8 +588,9 @@ class Bl(LongThumbInstruction):
 
 class Blx(ThumbInstruction):
     """ Branch with link with target in a register """
-    rm = Operand('rm', ArmRegister, read=True)
-    syntax = Syntax(['blx', ' ', rm])
+
+    rm = Operand("rm", ArmRegister, read=True)
+    syntax = Syntax(["blx", " ", rm])
 
     def encode(self):
         tokens = self.get_tokens()
@@ -481,24 +611,24 @@ class cond_base_ins(ThumbInstruction):
 
 
 def make_cond_branch(mnemonic, cond):
-    target = Operand('target', str)
+    target = Operand("target", str)
     syntax = Syntax([mnemonic, target])
-    members = {
-        'syntax': syntax, 'target': target, 'cond': cond}
-    return type(mnemonic + '_ins', (cond_base_ins,), members)
+    members = {"syntax": syntax, "target": target, "cond": cond}
+    return type(mnemonic + "_ins", (cond_base_ins,), members)
 
 
-Beq = make_cond_branch('beq', 0)
-Bne = make_cond_branch('bne', 1)
-Blt = make_cond_branch('blt', 0b1011)
-Ble = make_cond_branch('ble', 0b1101)
-Bgt = make_cond_branch('bgt', 0b1100)
-Bge = make_cond_branch('bge', 0b1010)
+Beq = make_cond_branch("beq", 0)
+Bne = make_cond_branch("bne", 1)
+Blt = make_cond_branch("blt", 0b1011)
+Ble = make_cond_branch("ble", 0b1101)
+Bgt = make_cond_branch("bgt", 0b1100)
+Bge = make_cond_branch("bge", 0b1010)
 
 
 # Long conditional jumps:
 class cond_base_ins_long(LongThumbInstruction):
     """ Encoding T3 """
+
     def encode(self):
         j1 = 0  # TODO: what do these mean?
         j2 = 0
@@ -511,23 +641,22 @@ class cond_base_ins_long(LongThumbInstruction):
 
 
 def make_long_cond_branch(mnemonic, cond):
-    target = Operand('target', str)
+    target = Operand("target", str)
     syntax = Syntax([mnemonic, target])
-    members = {
-        'syntax': syntax, 'target': target, 'cond': cond}
-    return type(mnemonic + '_ins', (cond_base_ins_long,), members)
+    members = {"syntax": syntax, "target": target, "cond": cond}
+    return type(mnemonic + "_ins", (cond_base_ins_long,), members)
 
 
-Beqw = make_long_cond_branch('beqw', 0)
-Bnew = make_long_cond_branch('bnew', 1)
-Bhsw = make_long_cond_branch('bhsw', 0b0010)  # unsigned higher or same
-Blow = make_long_cond_branch('blow', 0b0011)  # unsigned lower
-Bhiw = make_long_cond_branch('bhiw', 0b1000)  # unsigned higher
-Blsw = make_long_cond_branch('blsw', 0b1001)  # unsigned lower or same
-Bltw = make_long_cond_branch('bltw', 0b1011)  # signed less than
-Blew = make_long_cond_branch('blew', 0b1101)  # signed less than or equal
-Bgtw = make_long_cond_branch('bgtw', 0b1100)  # signed greater than
-Bgew = make_long_cond_branch('bgew', 0b1010)  # signed greater than or equal
+Beqw = make_long_cond_branch("beqw", 0)
+Bnew = make_long_cond_branch("bnew", 1)
+Bhsw = make_long_cond_branch("bhsw", 0b0010)  # unsigned higher or same
+Blow = make_long_cond_branch("blow", 0b0011)  # unsigned lower
+Bhiw = make_long_cond_branch("bhiw", 0b1000)  # unsigned higher
+Blsw = make_long_cond_branch("blsw", 0b1001)  # unsigned lower or same
+Bltw = make_long_cond_branch("bltw", 0b1011)  # signed less than
+Blew = make_long_cond_branch("blew", 0b1101)  # signed less than or equal
+Bgtw = make_long_cond_branch("bgtw", 0b1100)  # signed greater than
+Bgew = make_long_cond_branch("bgew", 0b1010)  # signed greater than or equal
 
 
 def push_bit_pos(n):
@@ -536,7 +665,7 @@ def push_bit_pos(n):
     elif n == 14:
         return 8
     else:  # pragma: no cover
-        raise NotImplementedError('not implemented for {}'.format(n))
+        raise NotImplementedError("not implemented for {}".format(n))
 
 
 def pop_bit_pos(n):
@@ -545,21 +674,21 @@ def pop_bit_pos(n):
     elif n == 15:
         return 8
     else:  # pragma: no cover
-        raise NotImplementedError('not implemented for {}'.format(n))
+        raise NotImplementedError("not implemented for {}".format(n))
 
 
 class Push(ThumbInstruction):
-    regs = Operand('regs', set)
-    syntax = Syntax(['push', ' ', regs])
+    regs = Operand("regs", set)
+    syntax = Syntax(["push", " ", regs])
 
     def __repr__(self):
-        return 'Push {{{}}}'.format(self.regs)
+        return "Push {{{}}}".format(self.regs)
 
     def encode(self):
         tokens = self.get_tokens()
         for n in register_numbers(self.regs):
             tokens[0][push_bit_pos(n)] = 1
-        tokens[0][9:16] = 0x5a
+        tokens[0][9:16] = 0x5A
         return tokens[0].encode()
 
 
@@ -569,11 +698,11 @@ def register_numbers(regs):
 
 
 class Pop(ThumbInstruction):
-    regs = Operand('regs', set)
-    syntax = Syntax(['pop', ' ', regs])
+    regs = Operand("regs", set)
+    syntax = Syntax(["pop", " ", regs])
 
     def __repr__(self):
-        return 'Pop {{{}}}'.format(self.regs)
+        return "Pop {{{}}}".format(self.regs)
 
     def encode(self):
         tokens = self.get_tokens()
@@ -584,15 +713,16 @@ class Pop(ThumbInstruction):
 
 
 class Yield(ThumbInstruction):
-    syntax = Syntax(['yield'])
+    syntax = Syntax(["yield"])
 
     def encode(self):
-        return u16(0xbf10)
+        return u16(0xBF10)
 
 
 class addspsp_base(ThumbInstruction):
     """ add/sub SP with imm7 << 2 """
-    imm7 = Operand('imm7', int)
+
+    imm7 = Operand("imm7", int)
 
     def encode(self):
         assert self.imm7 < 512
@@ -602,13 +732,15 @@ class addspsp_base(ThumbInstruction):
 
 class AddSp(addspsp_base):
     syntax = Syntax(
-        ['add', ' ', 'sp', ',', ' ', 'sp', ',', ' ', addspsp_base.imm7])
+        ["add", " ", "sp", ",", " ", "sp", ",", " ", addspsp_base.imm7]
+    )
     opcode = 0b101100000
 
 
 class SubSp(addspsp_base):
     syntax = Syntax(
-        ['sub', ' ', 'sp', ',', ' ', 'sp', ',', ' ', addspsp_base.imm7])
+        ["sub", " ", "sp", ",", " ", "sp", ",", " ", addspsp_base.imm7]
+    )
     opcode = 0b101100001
 
 
@@ -641,82 +773,83 @@ class SubSp(addspsp_base):
 #
 ###############
 
-@thumb_isa.pattern('stm', 'JMP', size=2)
+
+@thumb_isa.pattern("stm", "JMP", size=2)
 def pattern_jmp(context, tree):
     label = tree.value
     context.emit(Bw(label.name, jumps=[label]))
 
 
-@thumb_isa.pattern('reg', 'REGI32', size=0)
-@thumb_isa.pattern('reg', 'REGU32', size=0)
-@thumb_isa.pattern('reg', 'REGI16', size=0)
-@thumb_isa.pattern('reg', 'REGU16', size=0)
-@thumb_isa.pattern('reg', 'REGI8', size=0)
-@thumb_isa.pattern('reg', 'REGU8', size=0)
+@thumb_isa.pattern("reg", "REGI32", size=0)
+@thumb_isa.pattern("reg", "REGU32", size=0)
+@thumb_isa.pattern("reg", "REGI16", size=0)
+@thumb_isa.pattern("reg", "REGU16", size=0)
+@thumb_isa.pattern("reg", "REGI8", size=0)
+@thumb_isa.pattern("reg", "REGU8", size=0)
 def pattern_reg32(context, tree):
     return tree.value
 
 
-@thumb_isa.pattern('reg', 'I32TOU32(reg)', size=0)
-@thumb_isa.pattern('reg', 'U32TOI32(reg)', size=0)
-@thumb_isa.pattern('reg', 'I32TOI32(reg)', size=0)
-@thumb_isa.pattern('reg', 'U32TOU32(reg)', size=0)
+@thumb_isa.pattern("reg", "I32TOU32(reg)", size=0)
+@thumb_isa.pattern("reg", "U32TOI32(reg)", size=0)
+@thumb_isa.pattern("reg", "I32TOI32(reg)", size=0)
+@thumb_isa.pattern("reg", "U32TOU32(reg)", size=0)
 def pattern_i32toi32(context, tree, c0):
     return c0
 
 
-@thumb_isa.pattern('reg', 'I16TOI32(reg)', size=0)
-@thumb_isa.pattern('reg', 'U16TOI32(reg)', size=0)
-@thumb_isa.pattern('reg', 'I16TOU32(reg)', size=0)
-@thumb_isa.pattern('reg', 'U16TOU32(reg)', size=0)
+@thumb_isa.pattern("reg", "I16TOI32(reg)", size=0)
+@thumb_isa.pattern("reg", "U16TOI32(reg)", size=0)
+@thumb_isa.pattern("reg", "I16TOU32(reg)", size=0)
+@thumb_isa.pattern("reg", "U16TOU32(reg)", size=0)
 def pattern_i16_to_i32(context, tree, c0):
     # TODO: do something?
     return c0
 
 
-@thumb_isa.pattern('reg', 'I32TOI16(reg)', size=0)
-@thumb_isa.pattern('reg', 'I32TOU16(reg)', size=0)
-@thumb_isa.pattern('reg', 'U32TOI16(reg)', size=0)
-@thumb_isa.pattern('reg', 'U32TOU16(reg)', size=0)
+@thumb_isa.pattern("reg", "I32TOI16(reg)", size=0)
+@thumb_isa.pattern("reg", "I32TOU16(reg)", size=0)
+@thumb_isa.pattern("reg", "U32TOI16(reg)", size=0)
+@thumb_isa.pattern("reg", "U32TOU16(reg)", size=0)
 def pattern_i32toi16(context, tree, c0):
     # TODO: do something?
     return c0
 
 
-@thumb_isa.pattern('reg', 'I8TOI32(reg)', size=0)
-@thumb_isa.pattern('reg', 'U8TOI32(reg)', size=0)
-@thumb_isa.pattern('reg', 'I8TOU32(reg)', size=0)
-@thumb_isa.pattern('reg', 'U8TOU32(reg)', size=0)
+@thumb_isa.pattern("reg", "I8TOI32(reg)", size=0)
+@thumb_isa.pattern("reg", "U8TOI32(reg)", size=0)
+@thumb_isa.pattern("reg", "I8TOU32(reg)", size=0)
+@thumb_isa.pattern("reg", "U8TOU32(reg)", size=0)
 def pattern_i8toi32(context, tree, c0):
     # TODO: do something?
     return c0
 
 
-@thumb_isa.pattern('reg', 'I32TOI8(reg)', size=0)
-@thumb_isa.pattern('reg', 'I32TOU8(reg)', size=0)
-@thumb_isa.pattern('reg', 'U32TOI8(reg)', size=0)
-@thumb_isa.pattern('reg', 'U32TOU8(reg)', size=0)
+@thumb_isa.pattern("reg", "I32TOI8(reg)", size=0)
+@thumb_isa.pattern("reg", "I32TOU8(reg)", size=0)
+@thumb_isa.pattern("reg", "U32TOI8(reg)", size=0)
+@thumb_isa.pattern("reg", "U32TOU8(reg)", size=0)
 def pattern_i32toi8(context, tree, c0):
     # TODO: do something?
     return c0
 
 
-@thumb_isa.pattern('reg', 'ADDI32(reg,reg)', size=2)
-@thumb_isa.pattern('reg', 'ADDU32(reg,reg)', size=2)
+@thumb_isa.pattern("reg", "ADDI32(reg,reg)", size=2)
+@thumb_isa.pattern("reg", "ADDU32(reg,reg)", size=2)
 def pattern_add32(context, tree, c0, c1):
     d = context.new_reg(LowArmRegister)
     context.emit(Add3(d, c0, c1))
     return d
 
 
-@thumb_isa.pattern('reg', 'ADDI8(reg,reg)', size=1)
+@thumb_isa.pattern("reg", "ADDI8(reg,reg)", size=1)
 def pattern_add8(context, tree, c0, c1):
     d = context.new_reg(LowArmRegister)
     context.emit(Add3(d, c0, c1))
     return d
 
 
-@thumb_isa.pattern('reg', 'LABEL', size=2)
+@thumb_isa.pattern("reg", "LABEL", size=2)
 def pattern_label(context, tree):
     d = context.new_reg(LowArmRegister)
     ln = context.frame.add_constant(tree.value)
@@ -724,8 +857,7 @@ def pattern_label(context, tree):
     return d
 
 
-@thumb_isa.pattern(
-    'reg', 'FPRELU32', size=9, cycles=9)
+@thumb_isa.pattern("reg", "FPRELU32", size=9, cycles=9)
 def pattern_fprel32(context, tree):
     d = context.new_reg(LowArmRegister)
     c1 = tree.value.offset
@@ -757,10 +889,10 @@ def pattern_fprel32(context, tree):
     return d
 
 
-@thumb_isa.pattern('reg', 'CONSTI32', size=6, cycles=4, energy=4)
-@thumb_isa.pattern('reg', 'CONSTU32', size=6, cycles=4, energy=4)
-@thumb_isa.pattern('reg', 'CONSTI16', size=6, cycles=4, energy=4)
-@thumb_isa.pattern('reg', 'CONSTU16', size=6, cycles=4, energy=4)
+@thumb_isa.pattern("reg", "CONSTI32", size=6, cycles=4, energy=4)
+@thumb_isa.pattern("reg", "CONSTU32", size=6, cycles=4, energy=4)
+@thumb_isa.pattern("reg", "CONSTI16", size=6, cycles=4, energy=4)
+@thumb_isa.pattern("reg", "CONSTU16", size=6, cycles=4, energy=4)
 def pattern_const32(context, tree):
     d = context.new_reg(LowArmRegister)
     ln = context.frame.add_constant(tree.value)
@@ -769,11 +901,21 @@ def pattern_const32(context, tree):
 
 
 @thumb_isa.pattern(
-    'reg', 'CONSTU32', size=2, cycles=1, energy=1,
-    condition=lambda x: x.value in range(256))
+    "reg",
+    "CONSTU32",
+    size=2,
+    cycles=1,
+    energy=1,
+    condition=lambda x: x.value in range(256),
+)
 @thumb_isa.pattern(
-    'reg', 'CONSTI32', size=2, cycles=1, energy=1,
-    condition=lambda x: x.value in range(256))
+    "reg",
+    "CONSTI32",
+    size=2,
+    cycles=1,
+    energy=1,
+    condition=lambda x: x.value in range(256),
+)
 def pattern_const32_imm(context, tree):
     """ 8 bit constant loading """
     d = context.new_reg(LowArmRegister)
@@ -781,8 +923,8 @@ def pattern_const32_imm(context, tree):
     return d
 
 
-@thumb_isa.pattern('reg', 'CONSTI8', size=6, cycles=4, energy=4)
-@thumb_isa.pattern('reg', 'CONSTU8', size=6, cycles=4, energy=4)
+@thumb_isa.pattern("reg", "CONSTI8", size=6, cycles=4, energy=4)
+@thumb_isa.pattern("reg", "CONSTU8", size=6, cycles=4, energy=4)
 def pattern_const8(context, tree):
     d = context.new_reg(LowArmRegister)
     ln = context.frame.add_constant(tree.value)
@@ -791,11 +933,21 @@ def pattern_const8(context, tree):
 
 
 @thumb_isa.pattern(
-    'reg', 'CONSTI8', size=2, cycles=1, energy=1,
-    condition=lambda x: x.value in range(256))
+    "reg",
+    "CONSTI8",
+    size=2,
+    cycles=1,
+    energy=1,
+    condition=lambda x: x.value in range(256),
+)
 @thumb_isa.pattern(
-    'reg', 'CONSTU8', size=2, cycles=1, energy=1,
-    condition=lambda x: x.value in range(256))
+    "reg",
+    "CONSTU8",
+    size=2,
+    cycles=1,
+    energy=1,
+    condition=lambda x: x.value in range(256),
+)
 def pattern_const8_imm(context, tree):
     """ 8 bit constant loading """
     d = context.new_reg(LowArmRegister)
@@ -803,25 +955,25 @@ def pattern_const8_imm(context, tree):
     return d
 
 
-@thumb_isa.pattern('stm', 'MOVU32(reg)', size=2)
-@thumb_isa.pattern('stm', 'MOVI32(reg)', size=2)
+@thumb_isa.pattern("stm", "MOVU32(reg)", size=2)
+@thumb_isa.pattern("stm", "MOVI32(reg)", size=2)
 def pattern_mov32(context, tree, c0):
     reg = tree.value
     context.move(reg, c0)
 
 
-@thumb_isa.pattern('stm', 'MOVI16(reg)', size=2)
-@thumb_isa.pattern('stm', 'MOVU16(reg)', size=2)
-@thumb_isa.pattern('stm', 'MOVI8(reg)', size=2)
-@thumb_isa.pattern('stm', 'MOVU8(reg)', size=2)
+@thumb_isa.pattern("stm", "MOVI16(reg)", size=2)
+@thumb_isa.pattern("stm", "MOVU16(reg)", size=2)
+@thumb_isa.pattern("stm", "MOVI8(reg)", size=2)
+@thumb_isa.pattern("stm", "MOVU8(reg)", size=2)
 def pattern_mov8(context, tree, c0):
     reg = tree.value
     context.move(reg, c0)
 
 
-@thumb_isa.pattern('stm', 'CJMPI32(reg,reg)', size=6)
-@thumb_isa.pattern('stm', 'CJMPI16(reg,reg)', size=6)
-@thumb_isa.pattern('stm', 'CJMPI8(reg,reg)', size=6)
+@thumb_isa.pattern("stm", "CJMPI32(reg,reg)", size=6)
+@thumb_isa.pattern("stm", "CJMPI16(reg,reg)", size=6)
+@thumb_isa.pattern("stm", "CJMPI8(reg,reg)", size=6)
 def pattern_cjmp_signed(context, tree, c0, c1):
     op, yes_label, no_label = tree.value
     opnames = {"<": Bltw, ">": Bgtw, "==": Beqw, "!=": Bnew, ">=": Bgew}
@@ -832,15 +984,18 @@ def pattern_cjmp_signed(context, tree, c0, c1):
     context.emit(jmp_ins)
 
 
-@thumb_isa.pattern('stm', 'CJMPU32(reg,reg)', size=6)
-@thumb_isa.pattern('stm', 'CJMPU16(reg,reg)', size=6)
-@thumb_isa.pattern('stm', 'CJMPU8(reg,reg)', size=6)
+@thumb_isa.pattern("stm", "CJMPU32(reg,reg)", size=6)
+@thumb_isa.pattern("stm", "CJMPU16(reg,reg)", size=6)
+@thumb_isa.pattern("stm", "CJMPU8(reg,reg)", size=6)
 def pattern_cjmp_unsigned(context, tree, c0, c1):
     op, yes_label, no_label = tree.value
     opnames = {
-        "<": Blow, ">": Bhiw,
-        "==": Beqw, "!=": Bnew,
-        "<=": Blsw, ">=": Bhsw
+        "<": Blow,
+        ">": Bhiw,
+        "==": Beqw,
+        "!=": Bnew,
+        "<=": Blsw,
+        ">=": Bhsw,
     }
     Bop = opnames[op]
     jmp_ins = Bw(no_label.name, jumps=[no_label])
@@ -849,34 +1004,34 @@ def pattern_cjmp_unsigned(context, tree, c0, c1):
     context.emit(jmp_ins)
 
 
-@thumb_isa.pattern('mem', 'reg', size=0)
+@thumb_isa.pattern("mem", "reg", size=0)
 def pattern_mem_reg(context, tree, c0):
     return c0, 0
 
 
-@thumb_isa.pattern('stm', 'STRI8(mem,reg)', size=2)
-@thumb_isa.pattern('stm', 'STRU8(mem,reg)', size=2)
+@thumb_isa.pattern("stm", "STRI8(mem,reg)", size=2)
+@thumb_isa.pattern("stm", "STRU8(mem,reg)", size=2)
 def pattern_str8(context, tree, c0, c1):
     rg, offset = c0
     context.emit(Strb(c1, rg, offset))
 
 
-@thumb_isa.pattern('stm', 'STRI16(mem,reg)', size=2)
-@thumb_isa.pattern('stm', 'STRU16(mem,reg)', size=2)
+@thumb_isa.pattern("stm", "STRI16(mem,reg)", size=2)
+@thumb_isa.pattern("stm", "STRU16(mem,reg)", size=2)
 def pattern_str16(context, tree, c0, c1):
     rg, offset = c0
     context.emit(Strh(c1, rg, offset))
 
 
-@thumb_isa.pattern('stm', 'STRI32(mem, reg)', size=2)
-@thumb_isa.pattern('stm', 'STRU32(mem, reg)', size=2)
+@thumb_isa.pattern("stm", "STRI32(mem, reg)", size=2)
+@thumb_isa.pattern("stm", "STRU32(mem, reg)", size=2)
 def pattern_str32(context, tree, c0, c1):
     rg, offset = c0
     context.emit(Str2(c1, rg, offset))
 
 
-@thumb_isa.pattern('reg', 'LDRI8(mem)', size=2)
-@thumb_isa.pattern('reg', 'LDRU8(mem)', size=2)
+@thumb_isa.pattern("reg", "LDRI8(mem)", size=2)
+@thumb_isa.pattern("reg", "LDRU8(mem)", size=2)
 def pattern_ldr8(context, tree, c0):
     rg, offset = c0
     d = context.new_reg(LowArmRegister)
@@ -884,8 +1039,8 @@ def pattern_ldr8(context, tree, c0):
     return d
 
 
-@thumb_isa.pattern('reg', 'LDRI16(mem)', size=2)
-@thumb_isa.pattern('reg', 'LDRU16(mem)', size=2)
+@thumb_isa.pattern("reg", "LDRI16(mem)", size=2)
+@thumb_isa.pattern("reg", "LDRU16(mem)", size=2)
 def pattern_ldr16(context, tree, c0):
     rg, offset = c0
     d = context.new_reg(LowArmRegister)
@@ -893,8 +1048,8 @@ def pattern_ldr16(context, tree, c0):
     return d
 
 
-@thumb_isa.pattern('reg', 'LDRI32(mem)', size=2)
-@thumb_isa.pattern('reg', 'LDRU32(mem)', size=2)
+@thumb_isa.pattern("reg", "LDRI32(mem)", size=2)
+@thumb_isa.pattern("reg", "LDRU32(mem)", size=2)
 def pattern_ldr32(context, tree, c0):
     rg, offset = c0
     d = context.new_reg(LowArmRegister)
@@ -903,8 +1058,8 @@ def pattern_ldr32(context, tree, c0):
 
 
 # Arithmatic patterns:
-@thumb_isa.pattern('reg', 'SUBU32(reg,reg)', size=2)
-@thumb_isa.pattern('reg', 'SUBI32(reg,reg)', size=2)
+@thumb_isa.pattern("reg", "SUBU32(reg,reg)", size=2)
+@thumb_isa.pattern("reg", "SUBI32(reg,reg)", size=2)
 def pattern_sub32(context, tree, c0, c1):
     d = context.new_reg(LowArmRegister)
     context.emit(Sub3(d, c0, c1))
@@ -912,8 +1067,11 @@ def pattern_sub32(context, tree, c0, c1):
 
 
 @thumb_isa.pattern(
-    'reg', 'SUBI32(reg,CONSTI32)', size=2,
-    condition=lambda x: x[1].value in range(8))
+    "reg",
+    "SUBI32(reg,CONSTI32)",
+    size=2,
+    condition=lambda x: x[1].value in range(8),
+)
 def pattern_sub32_imm3(context, tree, c0):
     d = context.new_reg(LowArmRegister)
     c1 = tree[1].value
@@ -921,16 +1079,16 @@ def pattern_sub32_imm3(context, tree, c0):
     return d
 
 
-@thumb_isa.pattern('reg', 'SUBI8(reg,reg)', size=2)
+@thumb_isa.pattern("reg", "SUBI8(reg,reg)", size=2)
 def pattern_sub8(context, tree, c0, c1):
     d = context.new_reg(LowArmRegister)
     context.emit(Sub3(d, c0, c1))
     return d
 
 
-@thumb_isa.pattern('reg', 'SHRI8(reg, reg)', size=4)
-@thumb_isa.pattern('reg', 'SHRI16(reg, reg)', size=4)
-@thumb_isa.pattern('reg', 'SHRI32(reg, reg)', size=4)
+@thumb_isa.pattern("reg", "SHRI8(reg, reg)", size=4)
+@thumb_isa.pattern("reg", "SHRI16(reg, reg)", size=4)
+@thumb_isa.pattern("reg", "SHRI32(reg, reg)", size=4)
 def pattern_shr_i32(context, tree, c0, c1):
     d = context.new_reg(LowArmRegister)
     context.move(d, c0)
@@ -938,9 +1096,9 @@ def pattern_shr_i32(context, tree, c0, c1):
     return d
 
 
-@thumb_isa.pattern('reg', 'SHRU8(reg, reg)', size=4)
-@thumb_isa.pattern('reg', 'SHRU16(reg, reg)', size=4)
-@thumb_isa.pattern('reg', 'SHRU32(reg, reg)', size=4)
+@thumb_isa.pattern("reg", "SHRU8(reg, reg)", size=4)
+@thumb_isa.pattern("reg", "SHRU16(reg, reg)", size=4)
+@thumb_isa.pattern("reg", "SHRU32(reg, reg)", size=4)
 def pattern_shr_u32(context, tree, c0, c1):
     d = context.new_reg(LowArmRegister)
     context.move(d, c0)
@@ -948,24 +1106,24 @@ def pattern_shr_u32(context, tree, c0, c1):
     return d
 
 
-@thumb_isa.pattern('reg', 'NEGI32(reg)', size=2)
-@thumb_isa.pattern('reg', 'NEGU32(reg)', size=2)
-@thumb_isa.pattern('reg', 'NEGI16(reg)', size=2)
-@thumb_isa.pattern('reg', 'NEGU16(reg)', size=2)
-@thumb_isa.pattern('reg', 'NEGI8(reg)', size=2)
-@thumb_isa.pattern('reg', 'NEGU8(reg)', size=2)
+@thumb_isa.pattern("reg", "NEGI32(reg)", size=2)
+@thumb_isa.pattern("reg", "NEGU32(reg)", size=2)
+@thumb_isa.pattern("reg", "NEGI16(reg)", size=2)
+@thumb_isa.pattern("reg", "NEGU16(reg)", size=2)
+@thumb_isa.pattern("reg", "NEGI8(reg)", size=2)
+@thumb_isa.pattern("reg", "NEGU8(reg)", size=2)
 def pattern_neg32(context, tree, c0):
     d = context.new_reg(LowArmRegister)
     context.emit(Rsb(d, c0))
     return d
 
 
-@thumb_isa.pattern('reg', 'ORI32(reg, reg)', size=4)
-@thumb_isa.pattern('reg', 'ORU32(reg, reg)', size=4)
-@thumb_isa.pattern('reg', 'ORI16(reg, reg)', size=4)
-@thumb_isa.pattern('reg', 'ORU16(reg, reg)', size=4)
-@thumb_isa.pattern('reg', 'ORI8(reg, reg)', size=4)
-@thumb_isa.pattern('reg', 'ORU8(reg, reg)', size=4)
+@thumb_isa.pattern("reg", "ORI32(reg, reg)", size=4)
+@thumb_isa.pattern("reg", "ORU32(reg, reg)", size=4)
+@thumb_isa.pattern("reg", "ORI16(reg, reg)", size=4)
+@thumb_isa.pattern("reg", "ORU16(reg, reg)", size=4)
+@thumb_isa.pattern("reg", "ORI8(reg, reg)", size=4)
+@thumb_isa.pattern("reg", "ORU8(reg, reg)", size=4)
 def pattern_or32(context, tree, c0, c1):
     d = context.new_reg(LowArmRegister)
     context.move(d, c0)
@@ -973,12 +1131,12 @@ def pattern_or32(context, tree, c0, c1):
     return d
 
 
-@thumb_isa.pattern('reg', 'ANDI32(reg, reg)', size=4)
-@thumb_isa.pattern('reg', 'ANDU32(reg, reg)', size=4)
-@thumb_isa.pattern('reg', 'ANDI16(reg, reg)', size=4)
-@thumb_isa.pattern('reg', 'ANDU16(reg, reg)', size=4)
-@thumb_isa.pattern('reg', 'ANDI8(reg, reg)', size=4)
-@thumb_isa.pattern('reg', 'ANDU8(reg, reg)', size=4)
+@thumb_isa.pattern("reg", "ANDI32(reg, reg)", size=4)
+@thumb_isa.pattern("reg", "ANDU32(reg, reg)", size=4)
+@thumb_isa.pattern("reg", "ANDI16(reg, reg)", size=4)
+@thumb_isa.pattern("reg", "ANDU16(reg, reg)", size=4)
+@thumb_isa.pattern("reg", "ANDI8(reg, reg)", size=4)
+@thumb_isa.pattern("reg", "ANDU8(reg, reg)", size=4)
 def pattern_and32(context, tree, c0, c1):
     d = context.new_reg(LowArmRegister)
     context.move(d, c0)
@@ -986,12 +1144,12 @@ def pattern_and32(context, tree, c0, c1):
     return d
 
 
-@thumb_isa.pattern('reg', 'SHLI32(reg, reg)', size=4)
-@thumb_isa.pattern('reg', 'SHLU32(reg, reg)', size=4)
-@thumb_isa.pattern('reg', 'SHLI16(reg, reg)', size=4)
-@thumb_isa.pattern('reg', 'SHLU16(reg, reg)', size=4)
-@thumb_isa.pattern('reg', 'SHLI8(reg, reg)', size=4)
-@thumb_isa.pattern('reg', 'SHLU8(reg, reg)', size=4)
+@thumb_isa.pattern("reg", "SHLI32(reg, reg)", size=4)
+@thumb_isa.pattern("reg", "SHLU32(reg, reg)", size=4)
+@thumb_isa.pattern("reg", "SHLI16(reg, reg)", size=4)
+@thumb_isa.pattern("reg", "SHLU16(reg, reg)", size=4)
+@thumb_isa.pattern("reg", "SHLI8(reg, reg)", size=4)
+@thumb_isa.pattern("reg", "SHLU8(reg, reg)", size=4)
 def pattern_shl32(context, tree, c0, c1):
     d = context.new_reg(LowArmRegister)
     context.move(d, c0)
@@ -999,8 +1157,8 @@ def pattern_shl32(context, tree, c0, c1):
     return d
 
 
-@thumb_isa.pattern('reg', 'MULI32(reg, reg)', size=5)
-@thumb_isa.pattern('reg', 'MULU32(reg, reg)', size=5)
+@thumb_isa.pattern("reg", "MULI32(reg, reg)", size=5)
+@thumb_isa.pattern("reg", "MULU32(reg, reg)", size=5)
 def pattern_mul32(context, tree, c0, c1):
     d = context.new_reg(LowArmRegister)
     context.move(d, c0)
@@ -1009,14 +1167,14 @@ def pattern_mul32(context, tree, c0, c1):
     return d
 
 
-@thumb_isa.pattern('reg', 'DIVI32(reg, reg)', size=10)
+@thumb_isa.pattern("reg", "DIVI32(reg, reg)", size=10)
 def pattern_div32(context, tree, c0, c1):
     d = context.new_reg(LowArmRegister)
     context.emit(Sdiv(d, c0, c1))
     return d
 
 
-@thumb_isa.pattern('reg', 'REMI32(reg, reg)', size=6, cycles=10, energy=5)
+@thumb_isa.pattern("reg", "REMI32(reg, reg)", size=6, cycles=10, energy=5)
 def pattern_rem32(context, tree, c0, c1):
     d2 = context.new_reg(LowArmRegister)
     context.emit(Sdiv(d2, c0, c1))
@@ -1029,12 +1187,12 @@ def pattern_rem32(context, tree, c0, c1):
     return d
 
 
-@thumb_isa.pattern('reg', 'XORI32(reg, reg)', size=4)
-@thumb_isa.pattern('reg', 'XORU32(reg, reg)', size=4)
-@thumb_isa.pattern('reg', 'XORI16(reg, reg)', size=4)
-@thumb_isa.pattern('reg', 'XORU16(reg, reg)', size=4)
-@thumb_isa.pattern('reg', 'XORI8(reg, reg)', size=4)
-@thumb_isa.pattern('reg', 'XORU8(reg, reg)', size=4)
+@thumb_isa.pattern("reg", "XORI32(reg, reg)", size=4)
+@thumb_isa.pattern("reg", "XORU32(reg, reg)", size=4)
+@thumb_isa.pattern("reg", "XORI16(reg, reg)", size=4)
+@thumb_isa.pattern("reg", "XORU16(reg, reg)", size=4)
+@thumb_isa.pattern("reg", "XORI8(reg, reg)", size=4)
+@thumb_isa.pattern("reg", "XORU8(reg, reg)", size=4)
 def pattern_xor32(context, tree, c0, c1):
     d = context.new_reg(LowArmRegister)
     context.move(d, c0)
