@@ -8,9 +8,10 @@ from .instructions import stm8_isa
 
 
 class Stm8Arch(Architecture):
-    ''' STM8 architecture description. '''
-    name = 'stm8'
-    option_names = ('coolcc',)
+    """ STM8 architecture description. """
+
+    name = "stm8"
+    option_names = ("coolcc",)
 
     def __init__(self, options=None):
         super().__init__(options=options)
@@ -21,10 +22,15 @@ class Stm8Arch(Architecture):
 
         self.info = ArchInfo(
             type_infos={
-                ir.i8: TypeInfo(1, 1), ir.u8: TypeInfo(1, 1),
-                ir.i16: TypeInfo(2, 2), ir.u16: TypeInfo(2, 2),
-                'int': ir.i16, 'ptr': ir.u16
-            }, register_classes=registers.register_classes)
+                ir.i8: TypeInfo(1, 1),
+                ir.u8: TypeInfo(1, 1),
+                ir.i16: TypeInfo(2, 2),
+                ir.u16: TypeInfo(2, 2),
+                "int": ir.i16,
+                "ptr": ir.u16,
+            },
+            register_classes=registers.register_classes,
+        )
 
     def gen_prologue(self, frame):
         raise NotImplementedError()
@@ -42,25 +48,25 @@ class Stm8Arch(Architecture):
         raise NotImplementedError()
 
     def determine_arg_locations(self, arg_types):
-        ''' Calling convention in priority order:
+        """ Calling convention in priority order:
 
         - Pointers in index registers;
         - 16-bit variables in index registers;
         - 8-bit variables in accumulator register first, afterwards in index
           registers.
 
-        '''
+        """
         location = []
         live_in = set()
 
-        if self.has_option('coolcc'):
+        if self.has_option("coolcc"):
             # Better calling convention using more resisters
 
             accumulator_registers = [A]
             index_registers = [X, Y]
             # First check for pointers.
             for i in range(len(arg_types)):
-                if arg_types[i].name in ('ptr'):
+                if arg_types[i].name in ("ptr"):
                     if len(index_registers):
                         register = index_registers.pop()
                         location[i] = register
@@ -68,7 +74,7 @@ class Stm8Arch(Architecture):
 
             # Then check for 16-bit variables.
             for i in range(len(arg_types)):
-                if arg_types[i].name in ('int', 'i16', 'u16'):
+                if arg_types[i].name in ("int", "i16", "u16"):
                     if len(index_registers):
                         register = index_registers.pop()
                         location[i] = register
@@ -76,7 +82,7 @@ class Stm8Arch(Architecture):
 
             # Afterwards check for 8-bit variables.
             for i in range(len(arg_types)):
-                if arg_types[i].name in ('i8', 'u8'):
+                if arg_types[i].name in ("i8", "u8"):
                     if len(accumulator_registers) or len(index_registers):
                         if len(accumulator_registers):
                             register = accumulator_registers.pop()
@@ -87,11 +93,17 @@ class Stm8Arch(Architecture):
         else:
             # IAR-style calling convention:
             v8_registers = [
-                registers.vrb8, registers.vrb9,
-                registers.vrb10, registers.vrb11]
+                registers.vrb8,
+                registers.vrb9,
+                registers.vrb10,
+                registers.vrb11,
+            ]
             v16_registers = [
-                registers.vrw0, registers.vrw1,
-                registers.vrw2, registers.vrw3]
+                registers.vrw0,
+                registers.vrw1,
+                registers.vrw2,
+                registers.vrw3,
+            ]
 
             for arg_type in arg_types:
                 if arg_type in (ir.ptr, ir.i16, ir.u16):
@@ -100,7 +112,8 @@ class Stm8Arch(Architecture):
                     register = v8_registers.pop()
                 else:
                     raise NotImplementedError(
-                        'Argument type {} not implemented'.format(arg_type))
+                        "Argument type {} not implemented".format(arg_type)
+                    )
                 location.append(register)
                 live_in.add(register)
 
@@ -114,6 +127,7 @@ class Stm8Arch(Architecture):
             location = registers.vrw0
         else:
             raise NotImplementedError(
-                'Return type {} not implemented'.format(ret_type))
+                "Return type {} not implemented".format(ret_type)
+            )
         live_out.add(location)
         return location, tuple(live_out)
