@@ -47,11 +47,15 @@ from . import instructions, registers
 # TODO: Use something like the below?
 class WindowsCallingConvention(CallingConvention):
     """ Windows calling convention """
-    name = 'wincc'
+
+    name = "wincc"
     int_regs = [registers.rcx, registers.rdx, registers.r8, registers.r9]
     float_regs = [
-        registers.xmm0, registers.xmm1, registers.xmm2,
-        registers.xmm3]
+        registers.xmm0,
+        registers.xmm1,
+        registers.xmm2,
+        registers.xmm3,
+    ]
 
 
 class LinuxCallingConvention(CallingConvention):
@@ -76,35 +80,65 @@ class LinuxCallingConvention(CallingConvention):
 
     self.rv = rax
     """
-    name = 'sysv'
+
+    name = "sysv"
     locs = (
-        ([ir.i8, ir.i16, ir.i32, ir.i64, ir.u8, ir.u64, ir.ptr],
-         [registers.rdi, registers.rsi, registers.rdx,
-          registers.rcx, registers.r8, registers.r9]),
-        ([ir.f32, ir.f64],
-         [registers.xmm0, registers.xmm1, registers.xmm2,
-          registers.xmm3, registers.xmm4, registers.xmm5,
-          registers.xmm6, registers.xmm7]))
+        (
+            [ir.i8, ir.i16, ir.i32, ir.i64, ir.u8, ir.u64, ir.ptr],
+            [
+                registers.rdi,
+                registers.rsi,
+                registers.rdx,
+                registers.rcx,
+                registers.r8,
+                registers.r9,
+            ],
+        ),
+        (
+            [ir.f32, ir.f64],
+            [
+                registers.xmm0,
+                registers.xmm1,
+                registers.xmm2,
+                registers.xmm3,
+                registers.xmm4,
+                registers.xmm5,
+                registers.xmm6,
+                registers.xmm7,
+            ],
+        ),
+    )
 
 
 class X86_64Arch(Architecture):
     """ x86_64 architecture """
-    name = 'x86_64'
-    option_names = ('sse2', 'sse3', 'x87', 'wincc')
+
+    name = "x86_64"
+    option_names = ("sse2", "sse3", "x87", "wincc")
 
     def __init__(self, options=None):
         super().__init__(options=options)
         self.info = ArchInfo(
             type_infos={
-                ir.i8: TypeInfo(1, 1), ir.u8: TypeInfo(1, 1),
-                ir.i16: TypeInfo(2, 2), ir.u16: TypeInfo(2, 2),
-                ir.i32: TypeInfo(4, 4), ir.u32: TypeInfo(4, 4),
-                ir.i64: TypeInfo(8, 8), ir.u64: TypeInfo(8, 8),
-                'int': ir.i64, 'ptr': ir.u64, ir.ptr: ir.u64,
-            }, register_classes=register_classes)
+                ir.i8: TypeInfo(1, 1),
+                ir.u8: TypeInfo(1, 1),
+                ir.i16: TypeInfo(2, 2),
+                ir.u16: TypeInfo(2, 2),
+                ir.i32: TypeInfo(4, 4),
+                ir.u32: TypeInfo(4, 4),
+                ir.i64: TypeInfo(8, 8),
+                ir.u64: TypeInfo(8, 8),
+                ir.f32: TypeInfo(4, 4),
+                ir.f64: TypeInfo(8, 8),
+                "int": ir.i64,
+                "ptr": ir.u64,
+                ir.ptr: ir.u64,
+            },
+            register_classes=register_classes,
+        )
 
         self.isa = isa + data_isa + sse1_isa + sse2_isa
-        if self.has_option('x87'):
+        if self.has_option("x87"):
             # TODO: implement x87 isa also!
             self.isa = self.isa + x87_isa
         self.assembler = BaseAssembler()
@@ -114,38 +148,47 @@ class X86_64Arch(Architecture):
 
     def move(self, dst, src):
         """ Generate a move from src to dst """
-        if isinstance(dst, registers.Register8) and \
-                isinstance(src, registers.Register8):
+        if isinstance(dst, registers.Register8) and isinstance(
+            src, registers.Register8
+        ):
             return MovRegRm8(dst, RmReg8(src), ismove=True)
-        elif isinstance(dst, registers.Register16) and \
-                isinstance(src, registers.Register16):
+        elif isinstance(dst, registers.Register16) and isinstance(
+            src, registers.Register16
+        ):
             return bits16.MovRegRm(dst, RmReg16(src), ismove=True)
-        elif isinstance(dst, registers.Register32) and \
-                isinstance(src, registers.Register32):
+        elif isinstance(dst, registers.Register32) and isinstance(
+            src, registers.Register32
+        ):
             return bits32.MovRegRm(dst, RmReg32(src), ismove=True)
-        elif isinstance(dst, registers.Register64) and \
-                isinstance(src, registers.Register16):
+        elif isinstance(dst, registers.Register64) and isinstance(
+            src, registers.Register16
+        ):
             return instructions.MovsxRegRm16(dst, RmReg16(src), ismove=True)
-        elif isinstance(dst, registers.Register16) and \
-                isinstance(src, registers.Register64):
+        elif isinstance(dst, registers.Register16) and isinstance(
+            src, registers.Register64
+        ):
             # return instructions.MovsxRegRm16(dst, RmReg16(src), ismove=True)
             raise NotImplementedError()  # pragma: no cover
-        elif isinstance(dst, registers.Register8) and \
-                isinstance(src, registers.Register64):
+        elif isinstance(dst, registers.Register8) and isinstance(
+            src, registers.Register64
+        ):
             raise NotImplementedError()  # pragma: no cover
-        elif isinstance(dst, Register64) and \
-                isinstance(src, registers.Register8):
+        elif isinstance(dst, Register64) and isinstance(
+            src, registers.Register8
+        ):
             raise NotImplementedError()  # pragma: no cover
-        elif isinstance(dst, registers.Register64) and \
-                isinstance(src, registers.Register64):
+        elif isinstance(dst, registers.Register64) and isinstance(
+            src, registers.Register64
+        ):
             return bits64.MovRegRm(dst, RmReg64(src), ismove=True)
         elif isinstance(dst, XmmRegister) and isinstance(src, XmmRegister):
             return Movsd(dst, RmXmmRegDouble(src), ismove=True)
-        elif isinstance(dst, registers.XmmRegisterSingle) and \
-                isinstance(src, registers.XmmRegisterSingle):
+        elif isinstance(dst, registers.XmmRegisterSingle) and isinstance(
+            src, registers.XmmRegisterSingle
+        ):
             return Movss(dst, RmXmmRegSingle(src), ismove=True)
         else:  # pragma: no cover
-            raise NotImplementedError(str(type(dst))+str(type(src)))
+            raise NotImplementedError(str(type(dst)) + str(type(src)))
 
     def gen_memcpy(self, dst, src, count):
         """ Generate a memcpy action """
@@ -155,10 +198,11 @@ class X86_64Arch(Architecture):
         # Source pointer:
         yield instructions.Lea(rsi, src)
 
-        yield instructions.MovImm(rcx, count)     # Byte count
+        yield instructions.MovImm(rcx, count)  # Byte count
         yield instructions.Rep()
         yield RegisterUseDef(uses=(rcx,))
         yield instructions.Movsb()
+        yield RegisterUseDef(uses=(rdi, rsi))
 
         # for x in
         # Memcopy action!
@@ -213,7 +257,7 @@ class X86_64Arch(Architecture):
         // a in rcx, b in xmm1, c in r8 and d in xmm3
         """
         arg_locs = []
-        if self.has_option('wincc'):
+        if self.has_option("wincc"):
             # Windows calling convention:
             int_regs = [
                 (registers.rcx, registers.ecx),
@@ -252,17 +296,23 @@ class X86_64Arch(Architecture):
         for arg_type in arg_types:
             # Determine register:
             if arg_type in [
-                    ir.i8, ir.i64, ir.u8, ir.u64,
-                    ir.i16, ir.u16,
-                    ir.i32, ir.u32,  # TODO: maybe use eax and friends?
-                    ir.ptr]:
+                ir.i8,
+                ir.i64,
+                ir.u8,
+                ir.u64,
+                ir.i16,
+                ir.u16,
+                ir.i32,
+                ir.u32,  # TODO: maybe use eax and friends?
+                ir.ptr,
+            ]:
                 if int_regs:
                     if arg_type in [ir.i32, ir.u32]:
                         reg = int_regs.pop(0)[1]
                     else:
                         reg = int_regs.pop(0)[0]
 
-                    if self.has_option('wincc'):
+                    if self.has_option("wincc"):
                         float_regs.pop(0)
                 else:
                     # We need stack location!
@@ -277,7 +327,7 @@ class X86_64Arch(Architecture):
                     else:
                         reg = float_regs.pop(0)[1]
 
-                    if self.has_option('wincc'):
+                    if self.has_option("wincc"):
                         int_regs.pop(0)
                 else:
                     # We need stack location!
@@ -354,21 +404,25 @@ class X86_64Arch(Architecture):
             elif isinstance(arg_loc, StackLocation):
                 if isinstance(arg, registers.Register64):
                     yield bits64.MovRegRm(
-                        arg, RmMemDisp(rbp, stack_offset + 16))
+                        arg, RmMemDisp(rbp, stack_offset + 16)
+                    )
                     stack_offset += arg_loc.size
                 elif isinstance(arg, registers.Register32):
                     yield bits32.MovRegRm(
-                        arg, RmMemDisp(rbp, stack_offset + 16))
+                        arg, RmMemDisp(rbp, stack_offset + 16)
+                    )
                     stack_offset += arg_loc.size
                 elif isinstance(arg, StackLocation):
                     # Store memcpy action for later:
                     # cps.append((arg.offset, stack_offset, arg.size))
-                    raise NotImplementedError()
+                    # raise NotImplementedError(str(arg_loc))
+                    # Do not copy any incoming variable, it was copied during
+                    # call. Use memory as is!
                     stack_offset += arg.size
                 else:  # pragma: no cover
                     raise NotImplementedError()
             else:  # pragma: no cover
-                raise NotImplementedError('Parameters in memory not impl')
+                raise NotImplementedError("Parameters in memory not impl")
 
         for dst, src, count in cps:
             for instruction in self.gen_memcpy(dst, src, count):
@@ -384,15 +438,62 @@ class X86_64Arch(Architecture):
         yield RegisterUseDef(uses=live_out)
 
     def gen_call(self, frame, label, args, rv):
-        # def gen_fill_arguments(self, arg_types, args):
         """ This function moves arguments in the proper locations. """
-        arg_types = [a[0] for a in args]
-        arg_locs = self.determine_arg_locations(arg_types)
-        push_regs = []
 
         # Setup parameters:
+        arg_types = [a[0] for a in args]
+        arg_locs = self.determine_arg_locations(arg_types)
+        mem_args = []
+        reg_args = []
         for arg_loc, arg2 in zip(arg_locs, args):
             arg = arg2[1]
+            if isinstance(arg_loc, Register):
+                reg_args.append((arg_loc, arg))
+            elif isinstance(arg_loc, StackLocation):
+                if isinstance(arg, Register):
+                    mem_args.append((arg, 8))
+                elif isinstance(arg, StackLocation):
+                    mem_args.append((arg, arg.size))
+                    # raise NotImplementedError(str(arg))
+                    # memcpy(a, b, 100)
+                else:  # pragma: no cover
+                    raise NotImplementedError(str(arg))
+            else:  # pragma: no cover
+                raise NotImplementedError("Parameters in memory not impl")
+
+        # First fill values on stack:
+        # Pre align stack to 16 bytes:
+        stack_size = sum(p[1] for p in mem_args)
+        if stack_size % 16 != 0:
+            extra_padding = stack_size % 16
+            yield SubImm(rsp, extra_padding)
+            stack_size += extra_padding
+
+        # Push arguments in reverse order:
+        for pr in reversed(mem_args):
+            push_reg = pr[0]
+            if isinstance(push_reg, registers.Register64):
+                yield Push(push_reg)
+            elif isinstance(push_reg, registers.Register32):
+                yield self.move(registers.eax, push_reg)
+                yield RegisterUseDef(
+                    uses=(registers.eax,), defs=(registers.rax,)
+                )
+                yield Push(rax)
+            elif isinstance(push_reg, StackLocation):
+                # Invoke massive memcpy action!
+                # TODO: how about alignment?
+                yield SubImm(rsp, push_reg.size)
+                dst = instructions.RmMemDisp(rsp, 0)
+                src = instructions.RmMemDisp(rbp, push_reg.offset)
+                for memcpy_ins in self.gen_memcpy(dst, src, push_reg.size):
+                    yield memcpy_ins
+            else:  # pragma: no cover
+                raise NotImplementedError(str(push_reg))
+
+        # Next up, fill registers:
+        # Move register args to proper location:
+        for arg_loc, arg in reg_args:
             if isinstance(arg_loc, registers.Register64):
                 if isinstance(arg, registers.Register64):
                     yield self.move(arg_loc, arg)
@@ -404,8 +505,7 @@ class X86_64Arch(Architecture):
                 elif isinstance(arg, registers.Register16):
                     # Upcast char!
                     yield self.move(registers.ax, arg)
-                    yield instructions.MovsxRegRm16(
-                        rax, RmReg16(registers.ax))
+                    yield instructions.MovsxRegRm16(rax, RmReg16(registers.ax))
                     yield self.move(arg_loc, rax)
                 else:  # pragma: no cover
                     raise NotImplementedError()
@@ -420,37 +520,10 @@ class X86_64Arch(Architecture):
             elif isinstance(arg_loc, registers.XmmRegisterSingle):
                 assert isinstance(arg, registers.XmmRegisterSingle)
                 yield self.move(arg_loc, arg)
-            elif isinstance(arg_loc, StackLocation):
-                if isinstance(arg, Register):
-                    push_regs.append(arg)
-                elif isinstance(arg, StackLocation):
-                    raise NotImplementedError(str(arg))
-                    # memcpy(a, b, 100)
-                else:  # pragma: no cover
-                    raise NotImplementedError(str(arg))
             else:  # pragma: no cover
-                raise NotImplementedError('Parameters in memory not impl')
+                raise NotImplementedError(str(arg_loc))
 
-        # Pre align stack to 16 bytes:
-        stack_size = len(push_regs) * 8
-        if stack_size % 16 != 0:
-            extra_padding = stack_size % 16
-            yield SubImm(rsp, extra_padding)
-            stack_size += extra_padding
-
-        # Push arguments in reverse order:
-        for push_reg in reversed(push_regs):
-            if isinstance(push_reg, registers.Register64):
-                yield Push(push_reg)
-            elif isinstance(push_reg, registers.Register32):
-                yield self.move(registers.eax, push_reg)
-                yield RegisterUseDef(
-                    uses=(registers.eax,), defs=(registers.rax,))
-                yield Push(rax)
-            else:  # pragma: no cover
-                raise NotImplementedError(str(push_reg))
-
-        wincc = self.has_option('wincc')
+        wincc = self.has_option("wincc")
 
         # If windows, reserve space for rcx, rdx, r8 and r9:
         if wincc:
@@ -533,7 +606,7 @@ class X86_64Arch(Architecture):
                 for byte in value:
                     yield Db(byte)
             else:  # pragma: no cover
-                raise NotImplementedError('Constant of type {}'.format(value))
+                raise NotImplementedError("Constant of type {}".format(value))
 
 
 def round_up16(s, already_taken):
