@@ -22,13 +22,18 @@ sse2_isa = Isa()
 class SseInstruction(Instruction):
     tokens = [
         PrefixToken,
-        RexToken, OpcodeToken, ModRmToken, SibToken,
-        Imm8Token, Imm32Token]
+        RexToken,
+        OpcodeToken,
+        ModRmToken,
+        SibToken,
+        Imm8Token,
+        Imm32Token,
+    ]
 
     def set_user_patterns(self, tokens):
         # TODO: Improve this way of setting 'r':
-        tokens.set_field('r', (self.r.num & 8) >> 3)
-        tokens.set_field('reg', self.r.num & 0x7)
+        tokens.set_field("r", (self.r.num & 8) >> 3)
+        tokens.set_field("reg", self.r.num & 0x7)
 
     def encode(self):
         """ Use a custom encode... """
@@ -49,7 +54,7 @@ class SseInstruction(Instruction):
             r += tokens[1].encode()
 
         # 0f:
-        r += bytes([0x0f])
+        r += bytes([0x0F])
 
         # opcode:
         r += tokens[2].encode()
@@ -80,24 +85,27 @@ class SseInstruction(Instruction):
 
 class Sse1Instruction(SseInstruction):
     """ Sse1 instruction """
+
     isa = sse1_isa
 
 
 class Sse2Instruction(SseInstruction):
     """ Sse2 instruction """
+
     isa = sse2_isa
 
 
 class RmXmmReg(Constructor):
     """ Xmm register access """
-    reg_rm = Operand('reg_rm', XmmRegister, read=True)
+
+    reg_rm = Operand("reg_rm", XmmRegister, read=True)
     syntax = Syntax([reg_rm])
-    patterns = {'mod': 3}
+    patterns = {"mod": 3}
 
     def set_user_patterns(self, tokens):
         # TODO: Improve this way of setting 'r':
-        tokens.set_field('b', (self.reg_rm.num & 8) >> 3)
-        tokens.set_field('rm', self.reg_rm.num & 0x7)
+        tokens.set_field("b", (self.reg_rm.num & 8) >> 3)
+        tokens.set_field("rm", self.reg_rm.num & 0x7)
 
 
 RmXmmRegDouble = RmXmmReg
@@ -105,14 +113,15 @@ RmXmmRegDouble = RmXmmReg
 
 class RmXmmRegSingle(Constructor):
     """ Xmm register access """
-    reg_rm = Operand('reg_rm', XmmRegisterSingle, read=True)
+
+    reg_rm = Operand("reg_rm", XmmRegisterSingle, read=True)
     syntax = Syntax([reg_rm])
-    patterns = {'mod': 3}
+    patterns = {"mod": 3}
 
     def set_user_patterns(self, tokens):
         # TODO: Improve this way of setting 'r':
-        tokens.set_field('b', (self.reg_rm.num & 8) >> 3)
-        tokens.set_field('rm', self.reg_rm.num & 0x7)
+        tokens.set_field("b", (self.reg_rm.num & 8) >> 3)
+        tokens.set_field("rm", self.reg_rm.num & 0x7)
 
 
 xmm_rm_modes = (RmXmmReg, RmMem, RmMemDisp, RmAbs)
@@ -122,218 +131,245 @@ xmm_single_rm_modes = (RmXmmRegSingle, RmMem, RmMemDisp, RmAbs)
 
 class Movups(Sse1Instruction):
     """ Move unaligned packed single-fp values """
-    r = Operand('r', XmmRegister, write=True)
-    rm = Operand('rm', xmm_rm_modes)
-    syntax = Syntax(['movups', ' ', r, ',', ' ', rm])
-    patterns = {'opcode': 0x10}
+
+    r = Operand("r", XmmRegister, write=True)
+    rm = Operand("rm", xmm_rm_modes)
+    syntax = Syntax(["movups", " ", r, ",", " ", rm])
+    patterns = {"opcode": 0x10}
 
 
 class Movss(Sse1Instruction):
     """ Move scalar single-fp value """
-    r = Operand('r', XmmRegisterSingle, write=True)
-    rm = Operand('rm', xmm_single_rm_modes)
-    syntax = Syntax(['movss', ' ', r, ',', ' ', rm])
-    patterns = {'prefix': 0xf3, 'opcode': 0x10}
+
+    r = Operand("r", XmmRegisterSingle, write=True)
+    rm = Operand("rm", xmm_single_rm_modes)
+    syntax = Syntax(["movss", " ", r, ",", " ", rm])
+    patterns = {"prefix": 0xF3, "opcode": 0x10}
 
 
 class Movupd(Sse2Instruction):
     """ Move unaligned packed double-fp values """
-    r = Operand('r', XmmRegister, write=True)
-    rm = Operand('rm', xmm_rm_modes)
-    syntax = Syntax(['movupd', ' ', r, ',', ' ', rm])
-    patterns = {'prefix': 0x66, 'opcode': 0x10}
+
+    r = Operand("r", XmmRegister, write=True)
+    rm = Operand("rm", xmm_rm_modes)
+    syntax = Syntax(["movupd", " ", r, ",", " ", rm])
+    patterns = {"prefix": 0x66, "opcode": 0x10}
 
 
 class Movsd(Sse2Instruction):
     """ Move scalar double-fp value """
-    r = Operand('r', XmmRegisterDouble, write=True)
-    rm = Operand('rm', xmm_double_rm_modes)
-    syntax = Syntax(['movsd', ' ', r, ',', ' ', rm])
-    patterns = {'prefix': 0xf2, 'opcode': 0x10}
+
+    r = Operand("r", XmmRegisterDouble, write=True)
+    rm = Operand("rm", xmm_double_rm_modes)
+    syntax = Syntax(["movsd", " ", r, ",", " ", rm])
+    patterns = {"prefix": 0xF2, "opcode": 0x10}
 
 
 class Movss2(Sse1Instruction):
     """ Move scalar single-fp value """
-    rm = Operand('rm', xmm_single_rm_modes)
-    r = Operand('r', XmmRegisterSingle, read=True)
-    syntax = Syntax(['movss', ' ', rm, ',', ' ', r], priority=1)
-    patterns = {'prefix': 0xf3, 'opcode': 0x11}
+
+    rm = Operand("rm", xmm_single_rm_modes)
+    r = Operand("r", XmmRegisterSingle, read=True)
+    syntax = Syntax(["movss", " ", rm, ",", " ", r], priority=1)
+    patterns = {"prefix": 0xF3, "opcode": 0x11}
 
 
 class Movsd2(Sse2Instruction):
     """ Move scalar double-fp value """
-    rm = Operand('rm', xmm_double_rm_modes)
-    r = Operand('r', XmmRegisterDouble, read=True)
-    syntax = Syntax(['movsd', ' ', rm, ',', ' ', r], priority=1)
-    patterns = {'prefix': 0xf2, 'opcode': 0x11}
+
+    rm = Operand("rm", xmm_double_rm_modes)
+    r = Operand("r", XmmRegisterDouble, read=True)
+    syntax = Syntax(["movsd", " ", rm, ",", " ", r], priority=1)
+    patterns = {"prefix": 0xF2, "opcode": 0x11}
 
 
 class Addss(Sse1Instruction):
     """ Add scalar single-fp value """
-    r = Operand('r', XmmRegisterSingle, write=True)
-    rm = Operand('rm', xmm_single_rm_modes, read=True)
-    patterns = {'prefix': 0xf3, 'opcode': 0x58}
-    syntax = Syntax(['addss', ' ', r, ',', ' ', rm])
+
+    r = Operand("r", XmmRegisterSingle, write=True)
+    rm = Operand("rm", xmm_single_rm_modes, read=True)
+    patterns = {"prefix": 0xF3, "opcode": 0x58}
+    syntax = Syntax(["addss", " ", r, ",", " ", rm])
 
 
 class Addsd(Sse2Instruction):
     """ Add scalar double-fp value """
-    r = Operand('r', XmmRegisterDouble, write=True)
-    rm = Operand('rm', xmm_double_rm_modes, read=True)
-    patterns = {'prefix': 0xf2, 'opcode': 0x58}
-    syntax = Syntax(['addsd', ' ', r, ',', ' ', rm])
+
+    r = Operand("r", XmmRegisterDouble, write=True)
+    rm = Operand("rm", xmm_double_rm_modes, read=True)
+    patterns = {"prefix": 0xF2, "opcode": 0x58}
+    syntax = Syntax(["addsd", " ", r, ",", " ", rm])
 
 
 class Subss(Sse1Instruction):
     """ Sub scalar single-fp value """
-    r = Operand('r', XmmRegisterSingle, write=True)
-    rm = Operand('rm', xmm_single_rm_modes, read=True)
-    patterns = {'prefix': 0xf3, 'opcode': 0x5c}
-    syntax = Syntax(['subss', ' ', r, ',', ' ', rm])
+
+    r = Operand("r", XmmRegisterSingle, write=True)
+    rm = Operand("rm", xmm_single_rm_modes, read=True)
+    patterns = {"prefix": 0xF3, "opcode": 0x5C}
+    syntax = Syntax(["subss", " ", r, ",", " ", rm])
 
 
 class Subsd(Sse2Instruction):
     """ Sub scalar double-fp value """
-    r = Operand('r', XmmRegister, write=True)
-    rm = Operand('rm', xmm_double_rm_modes, read=True)
-    patterns = {'prefix': 0xf2, 'opcode': 0x5c}
-    syntax = Syntax(['subsd', ' ', r, ',', ' ', rm])
+
+    r = Operand("r", XmmRegister, write=True)
+    rm = Operand("rm", xmm_double_rm_modes, read=True)
+    patterns = {"prefix": 0xF2, "opcode": 0x5C}
+    syntax = Syntax(["subsd", " ", r, ",", " ", rm])
 
 
 class Mulss(Sse1Instruction):
     """ Multiply scalar single-fp value """
-    r = Operand('r', XmmRegisterSingle, write=True)
-    rm = Operand('rm', xmm_single_rm_modes, read=True)
-    patterns = {'prefix': 0xf3, 'opcode': 0x59}
-    syntax = Syntax(['mulss', ' ', r, ',', ' ', rm])
+
+    r = Operand("r", XmmRegisterSingle, write=True)
+    rm = Operand("rm", xmm_single_rm_modes, read=True)
+    patterns = {"prefix": 0xF3, "opcode": 0x59}
+    syntax = Syntax(["mulss", " ", r, ",", " ", rm])
 
 
 class Mulsd(Sse2Instruction):
     """ Multiply scalar double-fp value """
-    r = Operand('r', XmmRegisterDouble, write=True)
-    rm = Operand('rm', xmm_double_rm_modes, read=True)
-    patterns = {'prefix': 0xf2, 'opcode': 0x59}
-    syntax = Syntax(['mulsd', ' ', r, ',', ' ', rm])
+
+    r = Operand("r", XmmRegisterDouble, write=True)
+    rm = Operand("rm", xmm_double_rm_modes, read=True)
+    patterns = {"prefix": 0xF2, "opcode": 0x59}
+    syntax = Syntax(["mulsd", " ", r, ",", " ", rm])
 
 
 class Divss(Sse1Instruction):
     """ Divide scalar single-fp value """
-    r = Operand('r', XmmRegisterSingle, write=True)
-    rm = Operand('rm', xmm_single_rm_modes, read=True)
-    patterns = {'prefix': 0xf3, 'opcode': 0x5e}
-    syntax = Syntax(['divss', ' ', r, ',', ' ', rm])
+
+    r = Operand("r", XmmRegisterSingle, write=True)
+    rm = Operand("rm", xmm_single_rm_modes, read=True)
+    patterns = {"prefix": 0xF3, "opcode": 0x5E}
+    syntax = Syntax(["divss", " ", r, ",", " ", rm])
 
 
 class Divsd(Sse2Instruction):
     """ Divide scalar double-fp value """
-    r = Operand('r', XmmRegisterDouble, write=True)
-    rm = Operand('rm', xmm_double_rm_modes, read=True)
-    patterns = {'prefix': 0xf2, 'opcode': 0x5e}
-    syntax = Syntax(['divsd', ' ', r, ',', ' ', rm])
+
+    r = Operand("r", XmmRegisterDouble, write=True)
+    rm = Operand("rm", xmm_double_rm_modes, read=True)
+    patterns = {"prefix": 0xF2, "opcode": 0x5E}
+    syntax = Syntax(["divsd", " ", r, ",", " ", rm])
 
 
 class Cvtss2si(Sse1Instruction):
     """ Convert scalar single-fp to integer """
-    r = Operand('r', Register64, write=True)
-    rm = Operand('rm', xmm_single_rm_modes, read=True)
-    syntax = Syntax(['cvtss2si', ' ', r, ',', ' ', rm])
-    patterns = {'prefix': 0xf3, 'opcode': 0x2d, 'w': 1}
+
+    r = Operand("r", Register64, write=True)
+    rm = Operand("rm", xmm_single_rm_modes, read=True)
+    syntax = Syntax(["cvtss2si", " ", r, ",", " ", rm])
+    patterns = {"prefix": 0xF3, "opcode": 0x2D, "w": 1}
 
 
 class Cvtss2si_32(Sse1Instruction):
     """ Convert scalar single-fp to integer """
-    r = Operand('r', Register32, write=True)
-    rm = Operand('rm', xmm_single_rm_modes, read=True)
-    syntax = Syntax(['cvtss2si', ' ', r, ',', ' ', rm])
-    patterns = {'prefix': 0xf3, 'opcode': 0x2d, 'w': 0}
+
+    r = Operand("r", Register32, write=True)
+    rm = Operand("rm", xmm_single_rm_modes, read=True)
+    syntax = Syntax(["cvtss2si", " ", r, ",", " ", rm])
+    patterns = {"prefix": 0xF3, "opcode": 0x2D, "w": 0}
 
 
 class Cvtsd2si(Sse2Instruction):
     """ Convert scalar double-fp to integer """
-    r = Operand('r', Register64, write=True)
-    rm = Operand('rm', xmm_double_rm_modes, read=True)
-    syntax = Syntax(['cvtsd2si', ' ', r, ',', ' ', rm])
-    patterns = {'prefix': 0xf2, 'opcode': 0x2d, 'w': 1}
+
+    r = Operand("r", Register64, write=True)
+    rm = Operand("rm", xmm_double_rm_modes, read=True)
+    syntax = Syntax(["cvtsd2si", " ", r, ",", " ", rm])
+    patterns = {"prefix": 0xF2, "opcode": 0x2D, "w": 1}
 
 
 class Cvtsd2si_32(Sse2Instruction):
     """ Convert scalar double-fp to integer """
-    r = Operand('r', Register32, write=True)
-    rm = Operand('rm', xmm_double_rm_modes, read=True)
-    syntax = Syntax(['cvtsd2si', ' ', r, ',', ' ', rm])
-    patterns = {'prefix': 0xf2, 'opcode': 0x2d, 'w': 0}
+
+    r = Operand("r", Register32, write=True)
+    rm = Operand("rm", xmm_double_rm_modes, read=True)
+    syntax = Syntax(["cvtsd2si", " ", r, ",", " ", rm])
+    patterns = {"prefix": 0xF2, "opcode": 0x2D, "w": 0}
 
 
 class Cvtsi2ss(Sse1Instruction):
     """ Convert integer to scalar single-fp """
-    r = Operand('r', XmmRegisterSingle, write=True)
-    rm = Operand('rm', rm64_modes, read=True)
-    syntax = Syntax(['cvtsi2ss', ' ', r, ',', ' ', rm])
-    patterns = {'prefix': 0xf3, 'opcode': 0x2a, 'w': 1}
+
+    r = Operand("r", XmmRegisterSingle, write=True)
+    rm = Operand("rm", rm64_modes, read=True)
+    syntax = Syntax(["cvtsi2ss", " ", r, ",", " ", rm])
+    patterns = {"prefix": 0xF3, "opcode": 0x2A, "w": 1}
 
 
 class Cvtsi2ss_32(Sse1Instruction):
     """ Convert integer to scalar single-fp """
-    r = Operand('r', XmmRegisterSingle, write=True)
-    rm = Operand('rm', rm32_modes, read=True)
-    syntax = Syntax(['cvtsi2ss', ' ', r, ',', ' ', rm], priority=2)
-    patterns = {'prefix': 0xf3, 'opcode': 0x2a, 'w': 0}
+
+    r = Operand("r", XmmRegisterSingle, write=True)
+    rm = Operand("rm", rm32_modes, read=True)
+    syntax = Syntax(["cvtsi2ss", " ", r, ",", " ", rm], priority=2)
+    patterns = {"prefix": 0xF3, "opcode": 0x2A, "w": 0}
 
 
 class Cvtsi2sd(Sse2Instruction):
     """ Convert integer to scalar double-fp """
-    r = Operand('r', XmmRegisterDouble, write=True)
-    rm = Operand('rm', rm64_modes, read=True)
-    syntax = Syntax(['cvtsi2sd', ' ', r, ',', ' ', rm])
-    patterns = {'prefix': 0xf2, 'opcode': 0x2a, 'w': 1}
+
+    r = Operand("r", XmmRegisterDouble, write=True)
+    rm = Operand("rm", rm64_modes, read=True)
+    syntax = Syntax(["cvtsi2sd", " ", r, ",", " ", rm])
+    patterns = {"prefix": 0xF2, "opcode": 0x2A, "w": 1}
 
 
 class Cvtsi2sd_32(Sse2Instruction):
     """ Convert integer to scalar double-fp """
-    r = Operand('r', XmmRegisterDouble, write=True)
-    rm = Operand('rm', rm32_modes, read=True)
-    syntax = Syntax(['cvtsi2sd', ' ', r, ',', ' ', rm], priority=2)
-    patterns = {'prefix': 0xf2, 'opcode': 0x2a, 'w': 0}
+
+    r = Operand("r", XmmRegisterDouble, write=True)
+    rm = Operand("rm", rm32_modes, read=True)
+    syntax = Syntax(["cvtsi2sd", " ", r, ",", " ", rm], priority=2)
+    patterns = {"prefix": 0xF2, "opcode": 0x2A, "w": 0}
 
 
 class Cvtss2sd(Sse2Instruction):
     """ Convert scalar single-fp to scalar double-fp """
-    r = Operand('r', XmmRegisterDouble, write=True)
-    rm = Operand('rm', xmm_single_rm_modes, read=True)
-    syntax = Syntax(['cvtss2sd', ' ', r, ',', ' ', rm])
-    patterns = {'prefix': 0xf3, 'opcode': 0x5a}
+
+    r = Operand("r", XmmRegisterDouble, write=True)
+    rm = Operand("rm", xmm_single_rm_modes, read=True)
+    syntax = Syntax(["cvtss2sd", " ", r, ",", " ", rm])
+    patterns = {"prefix": 0xF3, "opcode": 0x5A}
 
 
 class Cvtsd2ss(Sse2Instruction):
     """ Convert scalar double-fp to scalar single-fp """
-    r = Operand('r', XmmRegisterSingle, write=True)
-    rm = Operand('rm', xmm_double_rm_modes, read=True)
-    syntax = Syntax(['cvtsd2ss', ' ', r, ',', ' ', rm])
-    patterns = {'prefix': 0xf2, 'opcode': 0x5a}
+
+    r = Operand("r", XmmRegisterSingle, write=True)
+    rm = Operand("rm", xmm_double_rm_modes, read=True)
+    syntax = Syntax(["cvtsd2ss", " ", r, ",", " ", rm])
+    patterns = {"prefix": 0xF2, "opcode": 0x5A}
 
 
 class Comiss(Sse1Instruction):
     """ Scalar compare single-fp values """
-    r = Operand('r', XmmRegisterSingle, read=True)
-    rm = Operand('rm', xmm_single_rm_modes, read=True)
-    syntax = Syntax(['comiss', ' ', r, ',', ' ', rm])
-    patterns = {'opcode': 0x2f}
+
+    r = Operand("r", XmmRegisterSingle, read=True)
+    rm = Operand("rm", xmm_single_rm_modes, read=True)
+    syntax = Syntax(["comiss", " ", r, ",", " ", rm])
+    patterns = {"opcode": 0x2F}
 
 
 class Ucomiss(Sse1Instruction):
     """ Unordered scalar compare single-fp values """
-    r = Operand('r', XmmRegisterSingle, read=True)
-    rm = Operand('rm', xmm_single_rm_modes, read=True)
-    syntax = Syntax(['ucomiss', ' ', r, ',', ' ', rm])
-    patterns = {'opcode': 0x2e}
+
+    r = Operand("r", XmmRegisterSingle, read=True)
+    rm = Operand("rm", xmm_single_rm_modes, read=True)
+    syntax = Syntax(["ucomiss", " ", r, ",", " ", rm])
+    patterns = {"opcode": 0x2E}
 
 
 class Ucomisd(Sse2Instruction):
     """ Unordered scalar compare double-fp values """
-    r = Operand('r', XmmRegisterDouble, read=True)
-    rm = Operand('rm', xmm_double_rm_modes, read=True)
-    syntax = Syntax(['ucomisd', ' ', r, ',', ' ', rm])
-    patterns = {'prefix': 0x66, 'opcode': 0x2e}
+
+    r = Operand("r", XmmRegisterDouble, read=True)
+    rm = Operand("rm", xmm_double_rm_modes, read=True)
+    syntax = Syntax(["ucomisd", " ", r, ",", " ", rm])
+    patterns = {"prefix": 0x66, "opcode": 0x2E}
 
 
 class SsePseudoInstruction(ArtificialInstruction):
@@ -341,8 +377,8 @@ class SsePseudoInstruction(ArtificialInstruction):
 
 
 class PushXmm(SsePseudoInstruction):
-    r = Operand('r', XmmRegister, read=True)
-    syntax = Syntax(['push', ' ', r])
+    r = Operand("r", XmmRegister, read=True)
+    syntax = Syntax(["push", " ", r])
 
     def render(self):
         # sub rsp, 16
@@ -353,8 +389,8 @@ class PushXmm(SsePseudoInstruction):
 
 
 class PopXmm(SsePseudoInstruction):
-    r = Operand('r', XmmRegister, write=True)
-    syntax = Syntax(['pop', ' ', r])
+    r = Operand("r", XmmRegister, write=True)
+    syntax = Syntax(["pop", " ", r])
 
     def render(self):
         # movdqu [rsp], r
@@ -364,14 +400,14 @@ class PopXmm(SsePseudoInstruction):
         yield AddImm(rsp, 16)
 
 
-@sse2_isa.pattern('regfp32', 'F64TOF32(regfp64)', size=6, cycles=3, energy=3)
+@sse2_isa.pattern("regfp32", "F64TOF32(regfp64)", size=6, cycles=3, energy=3)
 def pattern_f64tof32(context, tree, c0):
     dst = context.new_reg(XmmRegisterSingle)
     context.emit(Cvtsd2ss(dst, RmXmmRegDouble(c0)))
     return dst
 
 
-@sse2_isa.pattern('regfp64', 'F32TOF64(regfp32)', size=6, cycles=3, energy=3)
+@sse2_isa.pattern("regfp64", "F32TOF64(regfp32)", size=6, cycles=3, energy=3)
 def pattern_f32tof64(context, tree, c0):
     dst = context.new_reg(XmmRegisterDouble)
     context.emit(Cvtss2sd(dst, RmXmmRegSingle(c0)))
@@ -379,16 +415,16 @@ def pattern_f32tof64(context, tree, c0):
 
 
 # I64:
-@sse1_isa.pattern('reg64', 'F32TOI64(regfp32)', size=6, cycles=2, energy=2)
-@sse1_isa.pattern('reg64', 'F32TOU64(regfp32)', size=6, cycles=2, energy=2)
+@sse1_isa.pattern("reg64", "F32TOI64(regfp32)", size=6, cycles=2, energy=2)
+@sse1_isa.pattern("reg64", "F32TOU64(regfp32)", size=6, cycles=2, energy=2)
 def pattern_f32toi64(context, tree, c0):
     dst = context.new_reg(Register64)
     context.emit(Cvtss2si(dst, RmXmmRegSingle(c0)))
     return dst
 
 
-@sse2_isa.pattern('reg64', 'F64TOI64(regfp64)', size=6, cycles=3, energy=3)
-@sse2_isa.pattern('reg64', 'F64TOU64(regfp64)', size=6, cycles=3, energy=3)
+@sse2_isa.pattern("reg64", "F64TOI64(regfp64)", size=6, cycles=3, energy=3)
+@sse2_isa.pattern("reg64", "F64TOU64(regfp64)", size=6, cycles=3, energy=3)
 def pattern_f64toi64(context, tree, c0):
     dst = context.new_reg(Register64)
     context.emit(Cvtsd2si(dst, RmXmmRegDouble(c0)))
@@ -397,14 +433,14 @@ def pattern_f64toi64(context, tree, c0):
 
 # See also: https://stackoverflow.com/questions/11406654/
 # how-to-convert-an-unsigned-integer-to-floating-point-in-x86-32-bit-assembly
-@sse1_isa.pattern('regfp32', 'I64TOF32(reg64)', size=6, cycles=2, energy=2)
+@sse1_isa.pattern("regfp32", "I64TOF32(reg64)", size=6, cycles=2, energy=2)
 def pattern_i64tof32(context, tree, c0):
     dst = context.new_reg(XmmRegisterSingle)
     context.emit(Cvtsi2ss(dst, RmReg64(c0)))
     return dst
 
 
-@sse1_isa.pattern('regfp32', 'U64TOF32(reg64)', size=6, cycles=2, energy=2)
+@sse1_isa.pattern("regfp32", "U64TOF32(reg64)", size=6, cycles=2, energy=2)
 def pattern_u64tof32(context, tree, c0):
     dst = context.new_reg(XmmRegisterSingle)
     # Special case if value is bigger than 2**63.
@@ -430,14 +466,14 @@ def pattern_u64tof32(context, tree, c0):
     return dst
 
 
-@sse2_isa.pattern('regfp64', 'I64TOF64(reg64)', size=6, cycles=3, energy=3)
+@sse2_isa.pattern("regfp64", "I64TOF64(reg64)", size=6, cycles=3, energy=3)
 def pattern_i64tof64(context, tree, c0):
     dst = context.new_reg(XmmRegisterDouble)
     context.emit(Cvtsi2sd(dst, RmReg64(c0)))
     return dst
 
 
-@sse2_isa.pattern('regfp64', 'U64TOF64(reg64)', size=6, cycles=3, energy=3)
+@sse2_isa.pattern("regfp64", "U64TOF64(reg64)", size=6, cycles=3, energy=3)
 def pattern_u64tof64(context, tree, c0):
     dst = context.new_reg(XmmRegisterDouble)
     # Special case if value is bigger than 2**63.
@@ -464,30 +500,30 @@ def pattern_u64tof64(context, tree, c0):
 
 
 # I32:
-@sse1_isa.pattern('reg32', 'F32TOI32(regfp32)', size=6, cycles=2, energy=2)
-@sse1_isa.pattern('reg32', 'F32TOU32(regfp32)', size=6, cycles=2, energy=2)
+@sse1_isa.pattern("reg32", "F32TOI32(regfp32)", size=6, cycles=2, energy=2)
+@sse1_isa.pattern("reg32", "F32TOU32(regfp32)", size=6, cycles=2, energy=2)
 def pattern_f32toi32(context, tree, c0):
     dst = context.new_reg(Register32)
     context.emit(Cvtss2si_32(dst, RmXmmRegSingle(c0)))
     return dst
 
 
-@sse2_isa.pattern('reg32', 'F64TOI32(regfp64)', size=6, cycles=3, energy=3)
-@sse2_isa.pattern('reg32', 'F64TOU32(regfp64)', size=6, cycles=3, energy=3)
+@sse2_isa.pattern("reg32", "F64TOI32(regfp64)", size=6, cycles=3, energy=3)
+@sse2_isa.pattern("reg32", "F64TOU32(regfp64)", size=6, cycles=3, energy=3)
 def pattern_f64toi32(context, tree, c0):
     dst = context.new_reg(Register32)
     context.emit(Cvtsd2si_32(dst, RmXmmRegDouble(c0)))
     return dst
 
 
-@sse1_isa.pattern('regfp32', 'I32TOF32(reg32)', size=6, cycles=2, energy=2)
+@sse1_isa.pattern("regfp32", "I32TOF32(reg32)", size=6, cycles=2, energy=2)
 def pattern_i32tof32(context, tree, c0):
     dst = context.new_reg(XmmRegisterSingle)
     context.emit(Cvtsi2ss_32(dst, RmReg32(c0)))
     return dst
 
 
-@sse1_isa.pattern('regfp32', 'U32TOF32(reg32)', size=6, cycles=2, energy=2)
+@sse1_isa.pattern("regfp32", "U32TOF32(reg32)", size=6, cycles=2, energy=2)
 def pattern_u32tof32(context, tree, c0):
     dst = context.new_reg(XmmRegisterSingle)
 
@@ -504,14 +540,14 @@ def pattern_u32tof32(context, tree, c0):
     return dst
 
 
-@sse2_isa.pattern('regfp64', 'I32TOF64(reg32)', size=6, cycles=3, energy=3)
+@sse2_isa.pattern("regfp64", "I32TOF64(reg32)", size=6, cycles=3, energy=3)
 def pattern_i32tof64(context, tree, c0):
     dst = context.new_reg(XmmRegisterDouble)
     context.emit(Cvtsi2sd_32(dst, RmReg32(c0)))
     return dst
 
 
-@sse2_isa.pattern('regfp64', 'U32TOF64(reg32)', size=6, cycles=3, energy=3)
+@sse2_isa.pattern("regfp64", "U32TOF64(reg32)", size=6, cycles=3, energy=3)
 def pattern_u32tof64(context, tree, c0):
     dst = context.new_reg(XmmRegisterDouble)
 
@@ -528,9 +564,9 @@ def pattern_u32tof64(context, tree, c0):
     return dst
 
 
-@sse1_isa.pattern('regfp32', 'CONSTF32', size=8, cycles=6, energy=2)
+@sse1_isa.pattern("regfp32", "CONSTF32", size=8, cycles=6, energy=2)
 def pattern_const_f32(context, tree):
-    float_const = struct.pack('f', tree.value)
+    float_const = struct.pack("f", tree.value)
     const_label = context.frame.add_constant(float_const)
     label_addr = context.new_reg(Register64)
     context.emit(MovAdr(label_addr, const_label))
@@ -539,9 +575,9 @@ def pattern_const_f32(context, tree):
     return dst
 
 
-@sse2_isa.pattern('regfp64', 'CONSTF64', size=8, cycles=9, energy=4)
+@sse2_isa.pattern("regfp64", "CONSTF64", size=8, cycles=9, energy=4)
 def pattern_const_f64(context, tree):
-    float_const = struct.pack('d', tree.value)
+    float_const = struct.pack("d", tree.value)
     const_label = context.frame.add_constant(float_const)
     label_addr = context.new_reg(Register64)
     context.emit(MovAdr(label_addr, const_label))
@@ -550,10 +586,10 @@ def pattern_const_f64(context, tree):
     return dst
 
 
-@sse2_isa.pattern('regfp64', 'NEGF64(regfp64)', size=8, cycles=9, energy=4)
+@sse2_isa.pattern("regfp64", "NEGF64(regfp64)", size=8, cycles=9, energy=4)
 def pattern_neg_f64(context, tree, c0):
     """ Multiply by -1 """
-    float_const = struct.pack('d', -1)
+    float_const = struct.pack("d", -1)
     const_label = context.frame.add_constant(float_const)
     label_addr = context.new_reg(Register64)
     context.emit(MovAdr(label_addr, const_label))
@@ -563,10 +599,10 @@ def pattern_neg_f64(context, tree, c0):
     return dst
 
 
-@sse1_isa.pattern('regfp32', 'NEGF32(regfp32)', size=8, cycles=9, energy=4)
+@sse1_isa.pattern("regfp32", "NEGF32(regfp32)", size=8, cycles=9, energy=4)
 def pattern_neg_f32(context, tree, c0):
     """ Multiply by -1 """
-    float_const = struct.pack('f', -1)
+    float_const = struct.pack("f", -1)
     const_label = context.frame.add_constant(float_const)
     label_addr = context.new_reg(Register64)
     context.emit(MovAdr(label_addr, const_label))
@@ -577,7 +613,8 @@ def pattern_neg_f32(context, tree, c0):
 
 
 @sse1_isa.pattern(
-    'regfp32', 'ADDF32(regfp32, regfp32)', size=6, cycles=2, energy=2)
+    "regfp32", "ADDF32(regfp32, regfp32)", size=6, cycles=2, energy=2
+)
 def pattern_addf32(context, tree, c0, c1):
     dst = context.new_reg(XmmRegisterSingle)
     context.move(dst, c0)
@@ -586,7 +623,8 @@ def pattern_addf32(context, tree, c0, c1):
 
 
 @sse2_isa.pattern(
-    'regfp64', 'ADDF64(regfp64, regfp64)', size=6, cycles=4, energy=3)
+    "regfp64", "ADDF64(regfp64, regfp64)", size=6, cycles=4, energy=3
+)
 def pattern_addf64(context, tree, c0, c1):
     dst = context.new_reg(XmmRegisterDouble)
     context.emit(Movsd(dst, RmXmmRegDouble(c0)))
@@ -595,7 +633,8 @@ def pattern_addf64(context, tree, c0, c1):
 
 
 @sse1_isa.pattern(
-    'regfp32', 'SUBF32(regfp32, regfp32)', size=6, cycles=2, energy=2)
+    "regfp32", "SUBF32(regfp32, regfp32)", size=6, cycles=2, energy=2
+)
 def pattern_sub_f32(context, tree, c0, c1):
     dst = context.new_reg(XmmRegisterSingle)
     context.move(dst, c0)
@@ -604,7 +643,8 @@ def pattern_sub_f32(context, tree, c0, c1):
 
 
 @sse2_isa.pattern(
-    'regfp64', 'SUBF64(regfp64, regfp64)', size=6, cycles=3, energy=3)
+    "regfp64", "SUBF64(regfp64, regfp64)", size=6, cycles=3, energy=3
+)
 def pattern_sub_f64(context, tree, c0, c1):
     dst = context.new_reg(XmmRegisterDouble)
     context.emit(Movsd(dst, RmXmmRegDouble(c0)))
@@ -613,7 +653,8 @@ def pattern_sub_f64(context, tree, c0, c1):
 
 
 @sse1_isa.pattern(
-    'regfp32', 'MULF32(regfp32, regfp32)', size=6, cycles=2, energy=2)
+    "regfp32", "MULF32(regfp32, regfp32)", size=6, cycles=2, energy=2
+)
 def pattern_mul_f32(context, tree, c0, c1):
     dst = context.new_reg(XmmRegisterSingle)
     context.move(dst, c0)
@@ -622,7 +663,8 @@ def pattern_mul_f32(context, tree, c0, c1):
 
 
 @sse2_isa.pattern(
-    'regfp64', 'MULF64(regfp64, regfp64)', size=6, cycles=3, energy=3)
+    "regfp64", "MULF64(regfp64, regfp64)", size=6, cycles=3, energy=3
+)
 def pattern_mul_f64(context, tree, c0, c1):
     dst = context.new_reg(XmmRegisterDouble)
     context.emit(Movsd(dst, RmXmmRegDouble(c0)))
@@ -631,7 +673,8 @@ def pattern_mul_f64(context, tree, c0, c1):
 
 
 @sse1_isa.pattern(
-    'regfp32', 'DIVF32(regfp32, regfp32)', size=6, cycles=2, energy=2)
+    "regfp32", "DIVF32(regfp32, regfp32)", size=6, cycles=2, energy=2
+)
 def pattern_div_f32(context, tree, c0, c1):
     dst = context.new_reg(XmmRegisterSingle)
     context.move(dst, c0)
@@ -640,7 +683,8 @@ def pattern_div_f32(context, tree, c0, c1):
 
 
 @sse2_isa.pattern(
-    'regfp64', 'DIVF64(regfp64, regfp64)', size=6, cycles=4, energy=3)
+    "regfp64", "DIVF64(regfp64, regfp64)", size=6, cycles=4, energy=3
+)
 def pattern_div_f64(context, tree, c0, c1):
     dst = context.new_reg(XmmRegisterDouble)
     context.emit(Movsd(dst, RmXmmRegDouble(c0)))
@@ -648,49 +692,47 @@ def pattern_div_f64(context, tree, c0, c1):
     return dst
 
 
-@sse1_isa.pattern('stm', 'MOVF32(regfp32)', size=3, cycles=2, energy=2)
+@sse1_isa.pattern("stm", "MOVF32(regfp32)", size=3, cycles=2, energy=2)
 def pattern_mov_f32(context, tree, c0):
     context.move(tree.value, c0)
 
 
-@sse2_isa.pattern('stm', 'MOVF64(regfp64)', size=3, cycles=3, energy=3)
+@sse2_isa.pattern("stm", "MOVF64(regfp64)", size=3, cycles=3, energy=3)
 def pattern_mov_f64(context, tree, c0):
     context.emit(Movsd(tree.value, RmXmmRegDouble(c0)))
 
 
-@sse2_isa.pattern('regfp64', 'REGF64', size=0, cycles=0, energy=0)
-@sse1_isa.pattern('regfp32', 'REGF32', size=0, cycles=0, energy=0)
+@sse2_isa.pattern("regfp64", "REGF64", size=0, cycles=0, energy=0)
+@sse1_isa.pattern("regfp32", "REGF32", size=0, cycles=0, energy=0)
 def pattern_reg_fp(context, tree):
     return tree.value
 
 
-@sse1_isa.pattern('stm', 'STRF32(reg64, regfp32)', size=6, cycles=2, energy=2)
+@sse1_isa.pattern("stm", "STRF32(reg64, regfp32)", size=6, cycles=2, energy=2)
 def pattern_str_f32(context, tree, c0, c1):
     context.emit(Movss2(RmMem(c0), c1))
 
 
-@sse2_isa.pattern('stm', 'STRF64(reg64, regfp64)', size=6, cycles=3, energy=3)
+@sse2_isa.pattern("stm", "STRF64(reg64, regfp64)", size=6, cycles=3, energy=3)
 def pattern_str_f64(context, tree, c0, c1):
     context.emit(Movsd2(RmMem(c0), c1))
 
 
-@sse1_isa.pattern('regfp32', 'LDRF32(reg64)', size=6, cycles=2, energy=2)
+@sse1_isa.pattern("regfp32", "LDRF32(reg64)", size=6, cycles=2, energy=2)
 def pattern_ldr_f32(context, tree, c0):
     dst = context.new_reg(XmmRegisterSingle)
     context.emit(Movss(dst, RmMem(c0)))
     return dst
 
 
-@sse2_isa.pattern('regfp64', 'LDRF64(reg64)', size=6, cycles=3, energy=3)
+@sse2_isa.pattern("regfp64", "LDRF64(reg64)", size=6, cycles=3, energy=3)
 def pattern_ldr_f64(context, tree, c0):
     dst = context.new_reg(XmmRegisterDouble)
     context.emit(Movsd(dst, RmMem(c0)))
     return dst
 
 
-jump_opnames = {
-    "<": Jb, ">": Ja, "==": Je, "!=": Jne, ">=": Jae, '<=': Jbe
-}
+jump_opnames = {"<": Jb, ">": Ja, "==": Je, "!=": Jne, ">=": Jae, "<=": Jbe}
 
 
 def pattern_cjmp(context, value):
@@ -702,14 +744,16 @@ def pattern_cjmp(context, value):
 
 
 @sse1_isa.pattern(
-    'stm', 'CJMPF32(regfp32,regfp32)', size=6, cycles=3, energy=3)
+    "stm", "CJMPF32(regfp32,regfp32)", size=6, cycles=3, energy=3
+)
 def pattern_cjmp_f32(context, tree, c0, c1):
     context.emit(Ucomiss(c0, RmXmmRegSingle(c1)))
     pattern_cjmp(context, tree.value)
 
 
 @sse1_isa.pattern(
-    'stm', 'CJMPF64(regfp64,regfp64)', size=6, cycles=3, energy=3)
+    "stm", "CJMPF64(regfp64,regfp64)", size=6, cycles=3, energy=3
+)
 def pattern_cjmp_f64(context, tree, c0, c1):
     context.emit(Ucomisd(c0, RmXmmRegDouble(c1)))
     pattern_cjmp(context, tree.value)
