@@ -7,16 +7,11 @@ from .. import api
 from ..binutils.objectfile import ObjectFile
 
 
-parser = argparse.ArgumentParser(
-    description=__doc__, parents=[base_parser])
-parser.add_argument(
-    'input', help='input file', type=argparse.FileType('r'))
-parser.add_argument(
-    '--segment', '-S', help='segment to copy', required=True)
-parser.add_argument(
-    'output', help='output file')
-parser.add_argument(
-    '--output-format', '-O', help='output file format')
+parser = argparse.ArgumentParser(description=__doc__, parents=[base_parser])
+parser.add_argument("input", help="input file", type=argparse.FileType("r"))
+parser.add_argument("--segment", "-S", help="segment to copy", required=True)
+parser.add_argument("output", help="output file")
+parser.add_argument("--output-format", "-O", help="output file format")
 
 
 def objcopy(args=None):
@@ -29,5 +24,5 @@ def objcopy(args=None):
         api.objcopy(obj, args.segment, args.output_format, args.output)
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     objcopy()
