@@ -1,15 +1,23 @@
-
 import logging
 from ...irutils import Verifier
+from ...common import DiagnosticsManager
 from .context import Context
 from .lexer import Lexer
 from .parser import Parser
 from .codegenerator import CodeGenerator
 
 
+def pascal_to_ir(sources, march):
+    diag = DiagnosticsManager()
+    pascal_builder = PascalBuilder(diag, march.info)
+    ir_modules = pascal_builder.build(sources)
+    return ir_modules
+
+
 class PascalBuilder:
     """ Generates IR-code from pascal source. """
-    logger = logging.getLogger('pascal-builder')
+
+    logger = logging.getLogger("pascal-builder")
 
     def __init__(self, diag, arch_info):
         self.arch_info = arch_info
@@ -25,7 +33,7 @@ class PascalBuilder:
         Raises compiler error when something goes wrong.
         """
         assert isinstance(sources, (tuple, list))
-        self.logger.debug('Building %d sources', len(sources))
+        self.logger.debug("Building %d sources", len(sources))
 
         # Create a context where the modules can live:
         context = Context(self.arch_info)
@@ -43,7 +51,7 @@ class PascalBuilder:
         for ir_module in ir_modules:
             self.verifier.verify(ir_module)
 
-        self.logger.debug('build complete!')
+        self.logger.debug("build complete!")
         return ir_modules
 
     def do_parse(self, src, context):
