@@ -27,20 +27,19 @@ def read_wat(f) -> Module:
     return Module(wat)
 
 
-def wasmify(func, target='native'):
+def wasmify(func, target="native"):
     """ Convert a Python function to a WASM function, compiled
     to native code. Assumes that all variables are floats.
     Can be used as a decorator, like Numba!
     """
-    
+
     from ppci.lang.python import python_to_wasm
-    from ppci.wasm import instantiate
-    
+
     def f64_print(x: float) -> None:
         print(x)
-    
+
     wa = python_to_wasm(func)
-    imports = {'env':{'f64_print': f64_print}}
+    imports = {"env": {"f64_print": f64_print}}
     mod = instantiate(wa, imports, target=target)
     wasmfunc = getattr(mod.exports, func.__name__)
     return wasmfunc
